@@ -68,7 +68,8 @@ gda_connection_event_get_type (void)
 			NULL,
 			sizeof (GdaConnectionEvent),
 			0,
-			(GInstanceInitFunc) gda_connection_event_init
+			(GInstanceInitFunc) gda_connection_event_init,
+			0
 		};
 		g_static_mutex_lock (&registering);
 		if (type == 0)
@@ -102,7 +103,7 @@ gda_connection_event_class_init (GdaConnectionEventClass *klass)
 }
 
 static void
-gda_connection_event_init (GdaConnectionEvent *event, GdaConnectionEventClass *klass)
+gda_connection_event_init (GdaConnectionEvent *event, G_GNUC_UNUSED GdaConnectionEventClass *klass)
 {
 	event->priv = g_new0 (GdaConnectionEventPrivate, 1);
 	event->priv->type = GDA_CONNECTION_EVENT_ERROR;
