@@ -131,7 +131,8 @@ _gda_data_meta_wrapper_get_type (void)
 			NULL,
 			sizeof (GdaDataMetaWrapper),
 			0,
-			(GInstanceInitFunc) gda_data_meta_wrapper_init
+			(GInstanceInitFunc) gda_data_meta_wrapper_init,
+			0
 		};
 
 		static const GInterfaceInfo data_model_info = {
@@ -204,7 +205,7 @@ gda_data_meta_wrapper_data_model_init (GdaDataModelIface *iface)
 }
 
 static void
-gda_data_meta_wrapper_init (GdaDataMetaWrapper *model, GdaDataMetaWrapperClass *klass)
+gda_data_meta_wrapper_init (GdaDataMetaWrapper *model, G_GNUC_UNUSED GdaDataMetaWrapperClass *klass)
 {
 	g_return_if_fail (GDA_IS_DATA_META_WRAPPER (model));
 	model->priv = g_new0 (GdaDataMetaWrapperPrivate, 1);
@@ -300,7 +301,7 @@ gda_data_meta_wrapper_set_property (GObject *object,
 			break;
 		}
 		default:
-			g_assert_not_reached ();
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
 		}
 	}
@@ -321,7 +322,7 @@ gda_data_meta_wrapper_get_property (GObject *object,
 			g_value_set_object (value, G_OBJECT (model->priv->model));
 			break;
 		default:
-			g_assert_not_reached ();
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
 		}
 	}
@@ -331,13 +332,14 @@ gda_data_meta_wrapper_get_property (GObject *object,
  * gda_data_meta_wrapper_new
  * @model: a #GdaDataModel
  *
- * Creates a new #GdaDataModel object which buffers the rows of @model. This object is usefull
+ * Creates a new #GdaDataModel object which buffers the rows of @model. This object is useful
  * only if @model can only be metaed using cursor based method.
  *
  * Returns: a pointer to the newly created #GdaDataModel.
  */
 GdaDataModel *
-_gda_data_meta_wrapper_new (GdaDataModel *model, gboolean reuseable, gint *cols, gint size, GdaSqlIdentifierStyle mode,
+_gda_data_meta_wrapper_new (GdaDataModel *model, gboolean reusable, gint *cols,
+			    gint size, GdaSqlIdentifierStyle mode,
 			    GdaSqlReservedKeywordsFunc reserved_keyword_func)
 {
 	GdaDataMetaWrapper *retmodel;
@@ -347,13 +349,13 @@ _gda_data_meta_wrapper_new (GdaDataModel *model, gboolean reuseable, gint *cols,
 	retmodel = g_object_new (GDA_TYPE_DATA_META_WRAPPER,
 				 "model", model, NULL);
 			      
-	retmodel->priv->cols_to_wrap = g_new0 (gint, size);
-	memcpy (retmodel->priv->cols_to_wrap, cols, sizeof (gint) * size);
+	retmodel->priv->cols_to_wrap = g_new (gint, size);
+	memcpy (retmodel->priv->cols_to_wrap, cols, sizeof (gint) * size); /* Flawfinder: ignore */
 	retmodel->priv->cols_to_wrap_size = size;
 	retmodel->priv->mode = mode;
 	retmodel->priv->reserved_keyword_func = reserved_keyword_func;
 	
-	if (reuseable)
+	if (reusable)
 		retmodel->priv->computed_rows = g_hash_table_new_full (g_int_hash, g_int_equal,
 								       NULL, (GDestroyNotify) comp_row_free);
 	else {
