@@ -90,7 +90,8 @@ gda_data_model_get_type (void)
 			NULL,
 			0,
 			0,
-			(GInstanceInitFunc) NULL
+			(GInstanceInitFunc) NULL,
+			0
 		};
 		
 		g_static_rec_mutex_lock (&init_mutex);
@@ -104,7 +105,7 @@ gda_data_model_get_type (void)
 }
 
 static void
-gda_data_model_class_init (gpointer g_class)
+gda_data_model_class_init (G_GNUC_UNUSED gpointer g_class)
 {
 	static gboolean initialized = FALSE;
 
@@ -640,7 +641,7 @@ gda_data_model_get_value_at (GdaDataModel *model, gint col, gint row, GError **e
  *
  * Note: the same limitations and usage instructions apply as for gda_data_model_get_value_at().
  *
- * Upon errors FALSE will be returned and @error will be assigned a
+ * Upon errors %NULL will be returned and @error will be assigned a
  * #GError from the #GDA_DATA_MODEL_ERROR domain.
  *
  * Returns: a #GValue containing the value stored in the given
@@ -783,6 +784,12 @@ gda_data_model_set_values (GdaDataModel *model, gint row, GList *values, GError
  * gda_data_model_iter_move_next() (and gda_data_model_iter_move_prev() if
  * supported).
  *
+ * Note: for the #GdaDataProxy data model (which proxies any #GdaDataModel for modifications and
+ * has twice the number of columns of the proxied data model), this method will create an iterator
+ * in which only the columns of the proxied data model appear. If you need to have a #GdaDataModelIter
+ * in which all the proxy's columns appear, create it using:
+ * <programlisting><![CDATA[iter = g_object_new (GDA_TYPE_DATA_MODEL_ITER, "data-model", proxy, NULL);]]></programlisting>
+ *
  * Returns: (transfer full): a #GdaDataModelIter object, or %NULL if an error occurred
  */
 GdaDataModelIter *
@@ -1006,6 +1013,11 @@ static gchar *export_to_text_separated (GdaDataModel *model, const gint *cols, g
  * gda_data_model_export_to_file() documentation for more information about the @options argument (except for the
  * "OVERWRITE" option).
  *
+ * Warning: this function uses a #GdaDataModelIter iterator, and if @model does not offer a random access
+ * (check using gda_data_model_get_access_flags()), the iterator will be the same as normally used
+ * to access data in @model previously to calling this method, and this iterator will be moved (point to
+ * another row).
+ *
  * Returns: a new string.
  */
 gchar *
@@ -1154,6 +1166,8 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 							g_string_append_c (retstring, quote);
 						}
 						g_string_append_c (retstring, '\n');
+						if (!cols)
+							g_free (rcols);
 					}
 				}
 				else
@@ -1220,7 +1234,12 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
  *   <listitem><para>"INVALID_AS_NULL": a boolean value which, if set to %TRUE, considers any invalid data (for example for the date related values) as NULL</para></listitem>
  * </itemizedlist>
  *
- * Upon errors FALSE will be returned and @error will be assigned a
+ * Warning: this function uses a #GdaDataModelIter iterator, and if @model does not offer a random access
+ * (check using gda_data_model_get_access_flags()), the iterator will be the same as normally used
+ * to access data in @model previously to calling this method, and this iterator will be moved (point to
+ * another row).
+ *
+ * Upon errors %FALSE will be returned and @error will be assigned a
  * #GError from the #GDA_DATA_MODEL_ERROR domain.
  *
  * Returns: TRUE if no error occurred
@@ -1228,8 +1247,8 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 gboolean
 gda_data_model_export_to_file (GdaDataModel *model, GdaDataModelIOFormat format, 
 			       const gchar *file,
-			       const gint *cols, gint nb_cols, 
-			       const gint *rows, gint nb_rows, 
+			       const gint *cols, gint nb_cols,
+			       const gint *rows, gint nb_rows,
 			       GdaSet *options, GError **error)
 {
 	gchar *body;
@@ -1239,7 +1258,7 @@ gda_data_model_export_to_file (GdaDataModel *model, GdaDataModelIOFormat format,
 	g_return_val_if_fail (!options || GDA_IS_SET (options), FALSE);
 	g_return_val_if_fail (file, FALSE);
 
-	body = gda_data_model_export_to_string (model, format, cols, nb_cols, rows, nb_cols, options);
+	body = gda_data_model_export_to_string (model, format, cols, nb_cols, rows, nb_rows, options);
 	if (options) {
 		GdaHolder *holder;
 		
@@ -1272,31 +1291,50 @@ gda_data_model_export_to_file (GdaDataModel *model, GdaDataModelIOFormat format,
 }
 
 static gchar *
-export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols, const gint *rows, gint nb_rows, 
-			  gchar sep, gchar quote, gboolean field_quotes, gboolean null_as_empty, gboolean invalid_as_null)
+export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols,
+			  const gint *rows, gint nb_rows, 
+			  gchar sep, gchar quote, gboolean field_quotes,
+			  gboolean null_as_empty, gboolean invalid_as_null)
 {
 	GString *str;
-	gchar *retval;
-	gint c, nbrows, r;
+	gint c;
+	GdaDataModelIter *iter;
+	gboolean addnl = FALSE;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 
 	str = g_string_new ("");
-	if (!rows)
-		nbrows = gda_data_model_get_n_rows (model);
-	else
-		nbrows = nb_rows;
+	iter = gda_data_model_create_iter (model);
+	if (!iter)
+		return g_string_free (str, FALSE);
 
-	for (r = 0; r < nbrows; r++) {
-		if (r > 0)
+	if ((gda_data_model_iter_get_row (iter) == -1) && ! gda_data_model_iter_move_next (iter)) {
+		g_object_unref (iter);
+		return g_string_free (str, FALSE);
+	}
+
+	for (; gda_data_model_iter_is_valid (iter); gda_data_model_iter_move_next (iter)) {
+		if (rows) {
+			gint r;
+			for (r = 0; r < nb_rows; r++) { 
+				if (gda_data_model_iter_get_row (iter) == rows[r])
+					break;
+			}
+			if (r == nb_rows)
+				continue;
+		}
+		
+		if (addnl)
 			str = g_string_append_c (str, '\n');
+		else
+			addnl = TRUE;
 
 		for (c = 0; c < nb_cols; c++) {
 			GValue *value;
 			gchar *txt;
 
-			value = (GValue *) gda_data_model_get_value_at (model, cols[c], rows ? rows[r] : r, NULL);
-			if (invalid_as_null) {
+			value = (GValue*) gda_data_model_iter_get_value_at (iter, cols[c]);
+			if (value && invalid_as_null) {
 				if ((G_VALUE_TYPE (value) == G_TYPE_DATE)) {
 					GDate *date = (GDate*) g_value_get_boxed (value);
 					if (!g_date_valid (date))
@@ -1348,10 +1386,8 @@ export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols, c
 		}
 	}
 
-	retval = str->str;
-	g_string_free (str, FALSE);
-
-	return retval;
+	g_object_unref (iter);
+	return g_string_free (str, FALSE);
 }
 
 static void
@@ -1479,9 +1515,13 @@ find_column_from_id (GdaDataModel *model, const gchar *colid, gint *pos)
 
 	/* if no column has been found, assumr @colid is like "_%d" where %d is a column number */
 	if (!column && (*colid == '_')) {
-		column = gda_data_model_describe_column (model, atoi (colid + 1));
-		if (column)
-			*pos = atoi (colid + 1);
+		gint i;
+		i = atoi (colid + 1); /* Flawfinder: ignore */
+		if (i >= 0) {
+			column = gda_data_model_describe_column (model, i);
+			if (column)
+				*pos = i;
+		}
 	}
 
 	return column;
@@ -1493,7 +1533,7 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row, GError **error)
 	xmlNodePtr xml_field;
 	GList *value_list = NULL;
 	GPtrArray *values;
-	gint i;
+	gsize i;
 	gboolean retval = TRUE;
 	gint pos = 0;
 
@@ -2007,13 +2047,13 @@ gda_data_model_dump (GdaDataModel *model, FILE *to_stream)
 	if (!to_stream)
 		to_stream = stdout;
 
-	if (getenv ("GDA_DATA_MODEL_DUMP_ATTRIBUTES")) 
+	if (getenv ("GDA_DATA_MODEL_DUMP_ATTRIBUTES")) /* Flawfinder: ignore */
 		dump_attrs = TRUE;
-	if (getenv ("GDA_DATA_MODEL_DUMP_ROW_NUMBERS"))
+	if (getenv ("GDA_DATA_MODEL_DUMP_ROW_NUMBERS")) /* Flawfinder: ignore */
 		dump_rows = TRUE;
-	if (getenv ("GDA_DATA_MODEL_DUMP_TITLE")) 
+	if (getenv ("GDA_DATA_MODEL_DUMP_TITLE")) /* Flawfinder: ignore */
 		dump_title = TRUE;
-	if (getenv ("GDA_DATA_MODEL_NULL_AS_EMPTY")) 
+	if (getenv ("GDA_DATA_MODEL_NULL_AS_EMPTY")) /* Flawfinder: ignore */
 		null_as_empty = TRUE;
 
 	str = real_gda_data_model_dump_as_string (model, FALSE, dump_rows, dump_title, null_as_empty, &error);
@@ -2065,11 +2105,11 @@ gda_data_model_dump_as_string (GdaDataModel *model)
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 
-	if (getenv ("GDA_DATA_MODEL_DUMP_ROW_NUMBERS"))
+	if (getenv ("GDA_DATA_MODEL_DUMP_ROW_NUMBERS")) /* Flawfinder: ignore */
 		dump_rows = TRUE;
-	if (getenv ("GDA_DATA_MODEL_DUMP_TITLE")) 
+	if (getenv ("GDA_DATA_MODEL_DUMP_TITLE")) /* Flawfinder: ignore */
 		dump_title = TRUE;
-	if (getenv ("GDA_DATA_MODEL_NULL_AS_EMPTY")) 
+	if (getenv ("GDA_DATA_MODEL_NULL_AS_EMPTY")) /* Flawfinder: ignore */
 		null_as_empty = TRUE;
 
 	return real_gda_data_model_dump_as_string (model, FALSE, dump_rows, dump_title, null_as_empty, NULL);
@@ -2195,7 +2235,7 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 			if (! dump_attributes) {
 				value = gda_data_model_get_value_at (model, i, j, NULL);
 				if (!value) {
-					cols_size [i + col_offset] = MAX (cols_size [i + col_offset], strlen (ERROR_STRING));
+					cols_size [i + col_offset] = MAX ((guint)cols_size [i + col_offset], strlen (ERROR_STRING));
 				}
 				else {
 					str = NULL;
@@ -2230,7 +2270,7 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 		const gchar *title;
 		title = g_object_get_data (G_OBJECT (model), "name");
 		if (title) {
-			gint total_width = n_cols -1, i;
+			gsize total_width = n_cols -1;
 
 			for (i = 0; i < n_cols; i++)
 				total_width += cols_size [i];
@@ -2402,7 +2442,10 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 			g_strfreev (cols_str [i]);
 		g_free (cols_str);
 	}
-	g_string_append_printf (string, ngettext("(%d row)\n", "(%d rows)\n", n_rows), n_rows);
+	if (n_rows < 0)
+		g_string_append_printf (string, _("(error getting data)\n"));
+	else
+		g_string_append_printf (string, ngettext("(%d row)\n", "(%d rows)\n", n_rows), n_rows);
 
  out:
 	if (ramodel)
