@@ -27,12 +27,14 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <ctype.h>
 #include <string.h>
 #include <glib/gi18n-lib.h>
 #include <libgda/gda-log.h>
 #include <libgda/gda-util.h>
 #include <libgda/gda-server-provider.h>
 #include <libgda/gda-column.h>
+#include <libgda/gda-data-model-iter.h>
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
@@ -330,6 +332,11 @@ gda_utility_check_data_model (GdaDataModel *model, gint nbcols, ...)
  *
  * Dump the data in a #GdaDataModel into a xmlNodePtr (as used in libxml).
  *
+ * Warning: this function uses a #GdaDataModelIter iterator, and if @model does not offer a random access
+ * (check using gda_data_model_get_access_flags()), the iterator will be the same as normally used
+ * to access data in @model previously to calling this method, and this iterator will be moved (point to
+ * another row).
+ *
  * Returns: TRUE if no error occurred
  */
 gboolean
@@ -339,13 +346,14 @@ gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
 					 gboolean use_col_ids)
 {
 	gboolean retval = TRUE;
-	gint nrows, i;
 	gint *rcols, rnb_cols;
 	gchar **col_ids = NULL;
 	xmlNodePtr data = NULL;
+	GdaDataModelIter *iter;
 
 	/* compute columns if not provided */
 	if (!cols) {
+		gint i;
 		rnb_cols = gda_data_model_get_n_columns (model);
 		rcols = g_new (gint, rnb_cols);
 		for (i = 0; i < rnb_cols; i++)
@@ -376,23 +384,34 @@ gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
 	}
 
 	/* add the model data to the XML output */
-	if (!rows)
-		nrows = gda_data_model_get_n_rows (model);
-	else
-		nrows = nb_rows;
-	if (nrows > 0) {
+	iter = gda_data_model_create_iter (model);
+	if (iter && (gda_data_model_iter_get_row (iter) == -1) && ! gda_data_model_iter_move_next (iter)) {
+		g_object_unref (iter);
+		iter = NULL;
+	}
+	if (iter) {
 		xmlNodePtr row;
-		gint r, c;
-
+		
 		data = xmlNewChild (parent, NULL, (xmlChar*)"gda_array_data", NULL);
-		for (r = 0; (r < nrows) && retval; r++) {
+		for (; retval && gda_data_model_iter_is_valid (iter); gda_data_model_iter_move_next (iter)) {
+			gint c;
+			if (rows) {
+				gint r;
+				for (r = 0; r < nb_rows; r++) { 
+					if (gda_data_model_iter_get_row (iter) == rows[r])
+						break;
+				}
+				if (r == nb_rows)
+					continue;
+			}
+
 			row = xmlNewChild (data, NULL,  (xmlChar*)"gda_array_row", NULL);
 			for (c = 0; c < rnb_cols; c++) {
 				GValue *value;
 				gchar *str = NULL;
 				xmlNodePtr field = NULL;
 
-				value = (GValue *) gda_data_model_get_value_at (model, rcols [c], rows ? rows [r] : r, NULL);
+				value = (GValue*) gda_data_model_iter_get_value_at (iter, rcols[c]);
 				if (!value) {
 					retval = FALSE;
 					break;
@@ -435,10 +454,11 @@ gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
 				g_free (str);
 			}
 		}
+		g_object_unref (iter);
 	}
 
-	if(!cols)
-		g_free(rcols);
+	if (!cols)
+		g_free (rcols);
 
 	if (use_col_ids) {
 		gint c;
@@ -590,7 +610,7 @@ gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *
 			if (model) {
 				gint fno;
 				
-				fno = atoi (ptr2);
+				fno = atoi (ptr2); /* Flawfinder: ignore */
 				if ((fno < 0) ||
 				    (fno >= gda_data_model_get_n_columns (model))) 
 					g_warning (_("Field number %d not found in source named '%s'"), fno, ptr1); 
@@ -1071,8 +1091,6 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 			continue;
 		g_hash_table_insert (fields_hash, selfield->field_name, GINT_TO_POINTER (1));
 
-		gchar *str;
-		str = gda_sql_identifier_quote (selfield->field_name, cnc, NULL, FALSE, FALSE);
 		if (insert_stmt) {
 			GdaSqlField *field;
 			field = gda_sql_field_new (GDA_SQL_ANY_PART (ist));
@@ -1195,8 +1213,10 @@ gda_compute_select_statement_from_update (GdaStatement *update_stmt, GError **er
 	g_value_set_string ((target->expr->value = gda_value_new (G_TYPE_STRING)), ust->table->table_name);
 
 	/* WHERE */
-	sst->where_cond = gda_sql_expr_copy (ust->cond);
-	GDA_SQL_ANY_PART (sst->where_cond)->parent = GDA_SQL_ANY_PART (sst);
+	if (ust->cond) {
+		sst->where_cond = gda_sql_expr_copy (ust->cond);
+		GDA_SQL_ANY_PART (sst->where_cond)->parent = GDA_SQL_ANY_PART (sst);
+	}
 
 	gda_sql_statement_free (upd_stmt);
 
@@ -1571,7 +1591,7 @@ cmp_func (gconstpointer a, gconstpointer b)
 }
 
 /**
- * gda_completion_list_get
+ * gda_completion_list_get:
  * @cnc: a #GdaConnection object
  * @sql: a partial SQL statement which is the context of the completion proposal
  * @start: starting position within @sql of the "token" to complete (starts at 0)
@@ -1581,7 +1601,7 @@ cmp_func (gconstpointer a, gconstpointer b)
  * If no completion is available, then the returned array contains just one NULL entry, and
  * if it was not possible to try to compute a completions list, then %NULL is returned.
  *
- * Returns: a new array of strings, or %NULL (use g_strfreev() to free the returned array)
+ * Returns: (transfer full) (array zero-terminated=1) (allow-none): a new array of strings, or %NULL (use g_strfreev() to free the returned array)
  */
 gchar **
 gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint end)
@@ -1600,19 +1620,19 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 
 	/* init */
 	compl = g_array_new (TRUE, TRUE, sizeof (gchar *));
-	text = g_new0 (gchar, end - start + 2);
-	memcpy (text, sql + start, end - start + 1);
+	text = g_new (gchar, end - start + 2);
+	memcpy (text, sql + start, end - start + 1); /* Flawfinder: ignore */
 	text [end - start + 1] = 0;
 
 	if (start == 0) {
 		/* 
 		 * start of a statement => complete with SQL start of statement words 
 		 */
-		gint len;
-		gint i;
+		gsize len;
+		gsize i;
 		len = strlen (text);
 		for (i = 0; i < (sizeof (sql_start_words) / sizeof (gchar*)); i++) {
-			gint clen = strlen (sql_start_words[i]);
+			gsize clen = strlen (sql_start_words[i]);
 			if (!g_ascii_strncasecmp (sql_start_words[i], text, MIN (clen, len))) {
 				gchar *str;
 				str = g_strdup (sql_start_words[i]);
@@ -1764,11 +1784,11 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 	 * middle of a statement and no completion yet => complete with SQL statement words 
 	 */
 	{
-		gint len;
-		gint i;
+		gsize len;
+		gsize i;
 		len = strlen (text);
 		for (i = 0; i < (sizeof (sql_middle_words) / sizeof (gchar*)); i++) {
-			gint clen = strlen (sql_middle_words[i]);
+			gsize clen = strlen (sql_middle_words[i]);
 			if (!g_ascii_strncasecmp (sql_middle_words[i], text, MIN (clen, len))) {
 				gchar *str;
 				str = g_strdup (sql_middle_words[i]);
@@ -1782,7 +1802,7 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 	if (compl) {
 		if (compl->len >= 1) {
 			/* sort */
-			gint i;
+			gsize i;
 			g_array_sort (compl, cmp_func);
 
 			/* remove duplicates if any */
@@ -1824,17 +1844,17 @@ concat_ident (const char *prefix, const gchar *ident)
 
 	str = malloc (sizeof (char) * (plen + tlen + 1));
 	if (prefix) {
-		strcpy (str, prefix);
+		strcpy (str, prefix); /* Flawfinder: ignore */
 		str [plen - 1] = '.';
-		strcpy (str + plen, ident);
+		strcpy (str + plen, ident); /* Flawfinder: ignore */
 	}
 	else
-		strcpy (str, ident);
+		strcpy (str, ident); /* Flawfinder: ignore */
 	return str;
 }
 
 /**
- * gda_sql_identifier_split
+ * gda_sql_identifier_split:
  * @id: an SQL identifier
  * 
  * Splits @id into an array of it sub parts. @id's format has to be "&lt;part&gt;[.&lt;part&gt;[...]]" where
@@ -1843,7 +1863,7 @@ concat_ident (const char *prefix, const gchar *ident)
  *
  * For example the <![CDATA["test.\"ATable\""]]> string will result in the array: <![CDATA[{"test", "\"ATable\"", NULL}]]>
  *
- * Returns: a new %NULL-terminated array of strings, or NULL (use g_strfreev() to free the returned array)
+ * Returns: (transfer full) (array zero-terminated=1) (allow-none): a new %NULL-terminated array of strings, or NULL (use g_strfreev() to free the returned array)
  */
 gchar **
 gda_sql_identifier_split (const gchar *id)
@@ -2203,14 +2223,14 @@ static char rfc1738_reserved_chars[] =
  * <constant>&quot;%%ab&quot;</constant> where
  * <constant>ab</constant> is the hexadecimal number corresponding to the character.
  *
- * Returns: a new string
+ * Returns: (transfer full): a new string
  */
 gchar *
 gda_rfc1738_encode (const gchar *string)
 {
 	gchar *ret, *wptr;
 	const gchar *rptr;
-	gint i;
+	gsize i;
 
 	if (!string)
 		return NULL;
@@ -2253,7 +2273,7 @@ gda_rfc1738_encode (const gchar *string)
 		}
 
 		if (enc) {
-			sprintf (wptr, "%%%02x", (unsigned char) *rptr);
+			sprintf (wptr, "%%%02x", (unsigned char) *rptr); /* Flawfinder: ignore */
 			wptr += 3;
 		}
 		else {
@@ -2266,7 +2286,7 @@ gda_rfc1738_encode (const gchar *string)
 
 /**
  * gda_rfc1738_decode
- * @string: a string to encode 
+ * @string: a string to decode
  *
  * Decodes @string using the RFC 1738 recommendations: the
  * <constant>&lt;&gt;&quot;#%{}|\^~[]&apos;`;/?:@=&amp;</constant> and space characters are replaced by 
@@ -2463,40 +2483,50 @@ gda_connection_string_split (const gchar *string, gchar **out_cnc_params, gchar
 	if (*out_cnc_params) {
 		gchar *pos;
 
-		pos = g_strrstr (*out_cnc_params, "USERNAME=");
-		if (pos) {
-			for (ptr = pos + 9; ptr && *ptr != '\0' && *ptr != ';'; ptr++);
-			if (ptr != pos + 9)
-				*out_username = g_strndup (pos + 9, ptr - (pos + 9));
-
-			if (*ptr)
-				g_memmove (pos, ptr + 1, strlen (ptr));
-			else
-				*pos = 0;
-			gchar *tmp;
-			gint len;
-			tmp = *out_cnc_params;
-			len = strlen (tmp) - 1;
-			if (tmp [len] == ';')
-				tmp [len] = 0;
+		pos = strstr (*out_cnc_params, "USERNAME=");
+		while (pos) {
+			if (((pos > *out_cnc_params) && (*(pos-1) == ';')) ||
+			    (pos  == *out_cnc_params)) {
+				for (ptr = pos + 9; ptr && *ptr != '\0' && *ptr != ';'; ptr++);
+				if (ptr != pos + 9)
+					*out_username = g_strndup (pos + 9, ptr - (pos + 9));
+				
+				if (*ptr)
+					g_memmove (pos, ptr + 1, strlen (ptr));
+				else
+					*pos = 0;
+				gchar *tmp;
+				gint len;
+				tmp = *out_cnc_params;
+				len = strlen (tmp) - 1;
+				if (tmp [len] == ';')
+					tmp [len] = 0;
+				break;
+			}
+			pos = strstr (pos + 9, "USERNAME=");
 		}
 
-		pos = g_strrstr (*out_cnc_params, "PASSWORD=");
-		if (pos) {
-			for (ptr = pos + 9; ptr && *ptr != '\0' && *ptr != ';'; ptr++);
-			if (ptr != pos + 9)
-				*out_password = g_strndup (pos + 9, ptr - (pos + 9));
-
-			if (*ptr)
-				g_memmove (pos, ptr + 1, strlen (ptr));
-			else
-				*pos = 0;
-			gchar *tmp;
-			gint len;
-			tmp = *out_cnc_params;
-			len = strlen (tmp) - 1;
-			if (tmp [len] == ';')
-				tmp [len] = 0;
+		pos = strstr (*out_cnc_params, "PASSWORD=");
+		while (pos) {
+			if (((pos > *out_cnc_params) && (*(pos-1) == ';')) ||
+			    (pos  == *out_cnc_params)) {
+				for (ptr = pos + 9; ptr && *ptr != '\0' && *ptr != ';'; ptr++);
+				if (ptr != pos + 9)
+					*out_password = g_strndup (pos + 9, ptr - (pos + 9));
+				
+				if (*ptr)
+					g_memmove (pos, ptr + 1, strlen (ptr));
+				else
+					*pos = 0;
+				gchar *tmp;
+				gint len;
+				tmp = *out_cnc_params;
+				len = strlen (tmp) - 1;
+				if (tmp [len] == ';')
+					tmp [len] = 0;
+				break;
+			}
+			pos = strstr (pos +  9, "PASSWORD=");
 		}
 	}
 
@@ -2506,6 +2536,56 @@ gda_connection_string_split (const gchar *string, gchar **out_cnc_params, gchar
 	gda_rfc1738_decode (*out_password);
 }
 
+static gboolean
+_parse_iso8601_date (GDate *gdate, const gchar *value, char **out_endptr)
+{
+	GDateYear year;
+	GDateMonth month;
+	GDateDay day;
+	unsigned long int tmp;
+	char *endptr;
+
+	g_date_clear (gdate, 1);
+
+	if (! isdigit (*value))
+		return FALSE;
+	tmp = strtoul (value, &endptr, 10);
+	if (tmp <= G_MAXUINT16)
+		year = tmp;
+	else
+		return FALSE;
+	if (*endptr != '-')
+		return FALSE;
+
+	value = endptr + 1;
+	if (! isdigit (*value))
+		return FALSE;
+	tmp = strtoul (value, &endptr, 10);
+	month = tmp > 0 ? (tmp <= G_DATE_DECEMBER ? tmp : G_DATE_BAD_MONTH) : G_DATE_BAD_MONTH;
+	if (month == G_DATE_BAD_MONTH)
+		return FALSE;
+	if (*endptr != '-')
+		return FALSE;
+
+	value = endptr + 1;
+	if (! isdigit (*value))
+		return FALSE;
+	tmp = strtoul (value, &endptr, 10);
+	day = tmp > 0 ? (tmp <= G_MAXUINT8 ? tmp : G_DATE_BAD_DAY) : G_DATE_BAD_DAY;
+	if (day == G_DATE_BAD_DAY)
+		return FALSE;
+
+	if (g_date_valid_dmy (day, month, year)) {
+		g_date_set_dmy (gdate, day, month, year);
+		*out_endptr = endptr;
+		return TRUE;
+	}
+	else {
+		memset (gdate, 0, sizeof (GDate));
+		return FALSE;
+	}
+}
+
 /**
  * gda_parse_iso8601_date
  * @gdate: a pointer to a #GDate structure which will be filled
@@ -2513,30 +2593,94 @@ gda_connection_string_split (const gchar *string, gchar **out_cnc_params, gchar
  *
  * Extracts date parts from @value, and sets @gdate's contents
  *
- * Accepted date format is "YYYY-MM-DD".
+ * Accepted date format is "YYYY-MM-DD" (more or less than 4 digits for years and
+ * less than 2 digits for month and day are accepted). Years must be in the 1-65535 range,
+ * a limitation imposed by #GDate.
  *
- * Returns: TRUE if no error occurred
+ * Returns: %TRUE if no error occurred
  */
 gboolean
 gda_parse_iso8601_date (GDate *gdate, const gchar *value)
 {
-	GDateYear year;
-	GDateMonth month;
-	GDateDay day;
+	g_return_val_if_fail (gdate, FALSE);
 
-	year = atoi (value);
-	value += 5;
-	month = atoi (value);
-	value += 3;
-	day = atoi (value);
-	
-	g_date_clear (gdate, 1);
-	if (g_date_valid_dmy (day, month, year)) {
-		g_date_set_dmy (gdate, day, month, year);
-		return TRUE;
-	}
+	char *endptr;
+	if (!value)
+		return FALSE;
+
+	if (! _parse_iso8601_date (gdate, value, &endptr))
+		return FALSE;
+	if (*endptr)
+		return FALSE;
+	return TRUE;
+}
+
+static gboolean
+_parse_iso8601_time (GdaTime *timegda, const gchar *value, char **out_endptr)
+{
+	unsigned long int tmp;
+	char *endptr;
+
+	memset (timegda, 0, sizeof (GdaTime));
+	timegda->timezone = GDA_TIMEZONE_INVALID;
+
+	if (! isdigit (*value))
+		return FALSE;
+	tmp = strtoul (value, &endptr, 10);
+	if (tmp <= 23)
+		timegda->hour = tmp;
 	else
 		return FALSE;
+	if (*endptr != ':')
+		return FALSE;
+	
+	value = endptr + 1;
+	if (! isdigit (*value))
+		return FALSE;
+	tmp = strtoul (value, &endptr, 10);
+	if (tmp < 60)
+		timegda->minute = tmp;
+	else
+		return FALSE;
+	if (*endptr != ':')
+		return FALSE;
+
+	value = endptr + 1;
+	if (! isdigit (*value))
+		return FALSE;
+	tmp = strtoul (value, &endptr, 10);
+	if (tmp < 60)
+		timegda->second = tmp;
+	else
+		return FALSE;
+
+	if (*endptr && (*endptr != '.') && (*endptr != '+') && (*endptr != '-')) {
+		*out_endptr = endptr;
+		return TRUE; /* end of the parsing */
+	}
+
+	if (*endptr == '.') {
+		value = endptr + 1;
+		if (! isdigit (*value))
+			return FALSE;
+		tmp = strtoul (value, &endptr, 10);
+		if (tmp < G_MAXULONG)
+			timegda->fraction = tmp;
+		else
+			return FALSE;
+	}
+	if ((*endptr == '+') || (*endptr == '-')) {
+		long int stmp;
+		value = endptr;
+		stmp = strtol (value, &endptr, 10);
+		if ((stmp >= -24) && (stmp <= 24))
+			timegda->timezone = stmp * 60 * 60;
+		else
+			return FALSE;
+	}
+
+	*out_endptr = endptr;
+	return TRUE;
 }
 
 /**
@@ -2553,38 +2697,16 @@ gda_parse_iso8601_date (GDate *gdate, const gchar *value)
 gboolean
 gda_parse_iso8601_time (GdaTime *timegda, const gchar *value)
 {
-	timegda->hour = atoi (value);
-	value += 3;
-	timegda->minute = atoi (value);
-	value += 3;
-	timegda->second = atoi (value);
-	value += 2;
-	if (*value != '.') {
-		timegda->fraction = 0;
-	} else {
-		gint ndigits = 0;
-		gint64 fraction;
+	g_return_val_if_fail (timegda, FALSE);
 
-		value++;
-		fraction = atol (value);
-		while (*value && *value != '+') {
-			value++;
-			ndigits++;
-		}
-
-		while (fraction > 0 && ndigits > 3) {
-			fraction /= 10;
-			ndigits--;
-		}
-		
-		timegda->fraction = fraction;
-	}
-
-	if (*value)
-		timegda->timezone = atol (value) * 60 * 60;
-	else
-		timegda->timezone = 0;
+	char *endptr;
+	if (!value)
+		return FALSE;
 
+	if (! _parse_iso8601_time (timegda, value, &endptr))
+		return FALSE;
+	if (*endptr)
+		return FALSE;
 	return TRUE;
 }
 
@@ -2602,43 +2724,51 @@ gda_parse_iso8601_time (GdaTime *timegda, const gchar *value)
 gboolean
 gda_parse_iso8601_timestamp (GdaTimestamp *timestamp, const gchar *value)
 {
-	timestamp->year = atoi (value);
-	value += 5;
-	timestamp->month = atoi (value);
-	value += 3;
-	timestamp->day = atoi (value);
-	value += 3;
-	timestamp->hour = atoi (value);
-	value += 3;
-	timestamp->minute = atoi (value);
-	value += 3;
-	timestamp->second = atoi (value);
-	value += 2;
-	if (*value != '.') {
-		timestamp->fraction = 0;
-	} else {
-		gint ndigits = 0;
-		gint64 fraction;
+	g_return_val_if_fail (timestamp, FALSE);
 
-		value++;
-		fraction = atol (value);
-		while (*value && *value != '+') {
-			value++;
-			ndigits++;
-		}
+	gboolean retval = TRUE;
+	char *endptr;
+	GDate gdate;
+	GdaTime timegda;
 
-		while (fraction > 0 && ndigits > 3) {
-			fraction /= 10;
-			ndigits--;
-		}
-		
-		timestamp->fraction = fraction;
+	memset (timestamp, 0, sizeof (GdaTimestamp));
+	memset (&timegda, 0, sizeof (GdaTime));
+	timegda.timezone = GDA_TIMEZONE_INVALID;
+
+	if (!value)
+		return FALSE;
+
+	/* date part */
+	if (! _parse_iso8601_date (&gdate, value, &endptr)) {
+		retval = FALSE;
+		goto out;
 	}
+	timestamp->year = g_date_get_year (&gdate);
+	timestamp->month = g_date_get_month (&gdate);
+	timestamp->day = g_date_get_day (&gdate);
 
-	if (*value)
-		timestamp->timezone = atol (value) * 60 * 60;
-	else
-		timestamp->timezone = 0;
+	/* separator */
+	if (!*endptr)
+		goto out;
 
-	return TRUE;
+	if (*endptr != ' ') {
+		retval = FALSE;
+		goto out;
+	}
+	value = endptr + 1;
+	if (!*value)
+		goto out;
+
+	/* time part */
+	if (! _parse_iso8601_time (&timegda, value, &endptr) ||
+	    *endptr) 
+		retval = FALSE;
+ out:
+	timestamp->hour = timegda.hour;
+	timestamp->minute = timegda.minute;
+	timestamp->second = timegda.second;
+	timestamp->fraction = timegda.fraction;
+	timestamp->timezone = timegda.timezone;
+
+	return retval;
 }
