@@ -1,5 +1,5 @@
 /* GDA Thread provider
- * Copyright (C) 2009 The GNOME Foundation.
+ * Copyright (C) 2009 - 2010 The GNOME Foundation.
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -36,6 +36,8 @@
 #include "gda-thread-meta.h"
 #include "gda-thread-wrapper.h"
 #include "gda-thread-recordset.h"
+#include <libgda/sqlite/virtual/gda-virtual-provider.h>
+#include <libgda/sqlite/virtual/gda-virtual-connection.h>
 
 #define PROV_CLASS(provider) (GDA_SERVER_PROVIDER_CLASS (G_OBJECT_GET_CLASS (provider)))
 
@@ -125,6 +127,11 @@ static GObject             *gda_thread_provider_statement_execute (GdaServerProv
 								 gpointer cb_data, GError **error);
 static gboolean             gda_thread_provider_handle_async (GdaServerProvider *provider, GdaConnection *cnc, GError **error);
 
+/* Quoting */
+static gchar               *gda_thread_provider_identifier_quote (GdaServerProvider *provider, GdaConnection *cnc,
+								  const gchar *id,
+								  gboolean meta_store_convention, gboolean force_quotes);
+
 /* distributed transactions */
 static gboolean gda_thread_provider_xa_start    (GdaServerProvider *provider, GdaConnection *cnc, 
 						   const GdaXaTransactionId *xid, GError **error);
@@ -187,6 +194,7 @@ gda_thread_provider_class_init (GdaThreadProviderClass *klass)
 	provider_class->is_busy = NULL;
 	provider_class->cancel = NULL;
 	provider_class->create_connection = gda_thread_provider_create_connection;
+	provider_class->identifier_quote = gda_thread_provider_identifier_quote;
 
 	memset (&(provider_class->meta_funcs), 0, sizeof (GdaServerProviderMeta));
 	provider_class->meta_funcs._info = _gda_thread_meta__info;
@@ -231,6 +239,10 @@ gda_thread_provider_class_init (GdaThreadProviderClass *klass)
         provider_class->meta_funcs.routine_col = _gda_thread_meta_routine_col;
         provider_class->meta_funcs._routine_par = _gda_thread_meta__routine_par;
         provider_class->meta_funcs.routine_par = _gda_thread_meta_routine_par;
+        provider_class->meta_funcs._indexes_tab = _gda_thread_meta__indexes_tab;
+        provider_class->meta_funcs.indexes_tab = _gda_thread_meta_indexes_tab;
+        provider_class->meta_funcs._index_cols = _gda_thread_meta__index_cols;
+        provider_class->meta_funcs.index_cols = _gda_thread_meta_index_cols;
 
 	/* distributed transactions: if not supported, then provider_class->xa_funcs should be set to NULL */
 	provider_class->xa_funcs = g_new0 (GdaServerProviderXa, 1);
@@ -245,7 +257,7 @@ gda_thread_provider_class_init (GdaThreadProviderClass *klass)
 }
 
 static void
-gda_thread_provider_init (GdaThreadProvider *thread_prv, GdaThreadProviderClass *klass)
+gda_thread_provider_init (GdaThreadProvider *thread_prv, G_GNUC_UNUSED GdaThreadProviderClass *klass)
 {
 	thread_prv->priv = g_new0 (GdaThreadProviderPrivate, 1);
 	thread_prv->priv->prov_wrappers = g_hash_table_new_full (g_str_hash, g_str_equal,
@@ -267,7 +279,8 @@ _gda_thread_provider_get_type (void)
 			NULL, NULL,
 			sizeof (GdaThreadProvider),
 			0,
-			(GInstanceInitFunc) gda_thread_provider_init
+			(GInstanceInitFunc) gda_thread_provider_init,
+			0
 		};
 		g_static_mutex_lock (&registering);
 		if (type == 0)
@@ -278,12 +291,26 @@ _gda_thread_provider_get_type (void)
 	return type;
 }
 
+/*
+ * Check if the wrapped connection has been closed in the meanwhile
+ *
+ * Returns: %TRUE if @cnc is now also closed (WARNING: @cnc may not exist anymore it %TRUE is returned)
+ */
+static gboolean
+check_cnc_closed (GdaConnection *cnc, ThreadConnectionData *cdata)
+{
+	if (cdata->sub_connection_has_closed) {
+		gda_connection_close_no_warning (cnc);
+		return TRUE;
+	}
+	return FALSE;
+}
 
 /*
  * Get provider name request
  */
 static const gchar *
-gda_thread_provider_get_name (GdaServerProvider *provider)
+gda_thread_provider_get_name (G_GNUC_UNUSED GdaServerProvider *provider)
 {
 	return "ThreadWrapper";
 }
@@ -292,7 +319,7 @@ gda_thread_provider_get_name (GdaServerProvider *provider)
  * Get provider's version, no need to change this
  */
 static const gchar *
-gda_thread_provider_get_version (GdaServerProvider *provider)
+gda_thread_provider_get_version (G_GNUC_UNUSED GdaServerProvider *provider)
 {
 	return PACKAGE_VERSION;
 }
@@ -348,8 +375,9 @@ static void setup_signals (GdaConnection *cnc, ThreadConnectionData *cdata);
 
 static gboolean
 gda_thread_provider_open_connection (GdaServerProvider *provider, GdaConnection *cnc,
-				     GdaQuarkList *params, GdaQuarkList *auth,
-				     guint *task_id, GdaServerProviderAsyncCallback async_cb, gpointer cb_data)
+				     GdaQuarkList *params, G_GNUC_UNUSED GdaQuarkList *auth,
+				     G_GNUC_UNUSED guint *task_id, GdaServerProviderAsyncCallback async_cb,
+				     G_GNUC_UNUSED gpointer cb_data)
 {
 	GdaThreadWrapper *wr = NULL;
 	gboolean wr_created = FALSE;
@@ -363,7 +391,10 @@ gda_thread_provider_open_connection (GdaServerProvider *provider, GdaConnection
 						 _("Provider does not support asynchronous connection open"));
                 return FALSE;
 	}
-		
+
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+	g_static_mutex_lock (&mutex);
+
 	/* test if connection has to be opened using a DSN or a connection string */
 	gchar *dsn, *auth_string, *cnc_string;
 	GdaConnectionOptions options;
@@ -391,14 +422,6 @@ gda_thread_provider_open_connection (GdaServerProvider *provider, GdaConnection
 					  data->prov_name);
 	}
 	g_assert (data);
-	
-	/* open sub connection */
-	GdaConnection *sub_cnc;
-	GError *error = NULL;
-	guint jid;
-	g_assert (data);
-	data->auth_string = auth_string;
-	data->options = options & (~GDA_CONNECTION_OPTIONS_THREAD_SAFE);
 
 	if (!wr) {
 		wr_created = TRUE;
@@ -406,11 +429,24 @@ gda_thread_provider_open_connection (GdaServerProvider *provider, GdaConnection
 		if (!wr) {
 			gda_connection_add_event_string (cnc, "%s", _("Multi threading is not supported or enabled"));
 			g_free (data);
+			g_static_mutex_unlock (&mutex);
 			return FALSE;
 		}
 	}
-	else
+	else {
+		/* here wr_created == FALSE */
 		g_object_ref (wr);
+		g_static_mutex_unlock (&mutex);
+	}
+	
+	/* open sub connection */
+	GdaConnection *sub_cnc;
+	GError *error = NULL;
+	guint jid;
+	g_assert (data);
+	data->auth_string = auth_string;
+	data->options = options & (~(GDA_CONNECTION_OPTIONS_THREAD_ISOLATED | GDA_CONNECTION_OPTIONS_THREAD_SAFE));
+
 	jid = gda_thread_wrapper_execute (wr, (GdaThreadWrapperFunc) sub_thread_open_connection, data, NULL, NULL);
 	sub_cnc = gda_thread_wrapper_fetch_result (wr, TRUE, jid, &error);
 	g_free (dsn);
@@ -422,33 +458,82 @@ gda_thread_provider_open_connection (GdaServerProvider *provider, GdaConnection
 			g_error_free (error);
 		g_object_unref (wr);
 		g_free (data);
+		if (wr_created)
+			g_static_mutex_unlock (&mutex);
 		return FALSE;
 	}
 	
 	ThreadConnectionData *cdata;
 	cdata = g_new0 (ThreadConnectionData, 1);
 	cdata->sub_connection = sub_cnc;
-	cdata->cnc_provider = data->out_cnc_provider;
+	cdata->sub_connection_has_closed = FALSE;
+	cdata->cnc_provider = g_object_ref (data->out_cnc_provider);
 	cdata->wrapper = wr;
 	cdata->handlers_ids = g_array_sized_new (FALSE, FALSE, sizeof (gulong), 2);
 	g_free (data);
 	gda_connection_internal_set_provider_data (cnc, cdata, (GDestroyNotify) gda_thread_free_cnc_data);
 	setup_signals (cnc, cdata);
 
-	if (wr_created && PROV_CLASS (cdata->cnc_provider)->limiting_thread) {
-		/* keep GdaThreadWrapper for other uses */
-		g_hash_table_insert (GDA_THREAD_PROVIDER (provider)->priv->prov_wrappers,
-				     g_strdup (gda_server_provider_get_name (cdata->cnc_provider)),
-				     g_object_ref (wr));
+	if (wr_created) {
+		if (PROV_CLASS (cdata->cnc_provider)->limiting_thread) {
+			/* keep GdaThreadWrapper for other uses */
+			g_hash_table_insert (GDA_THREAD_PROVIDER (provider)->priv->prov_wrappers,
+					     g_strdup (gda_server_provider_get_name (cdata->cnc_provider)),
+					     g_object_ref (wr));
+		}
+		g_static_mutex_unlock (&mutex);
 	}
 
 	return TRUE;
 }
 
+/**
+ * _gda_thread_provider_handle_virtual_connection
+ * @provider: a #GdaThreadProvider provider
+ * @sub_cnc: a #GdaConnection which has already been opened, normally a virtual connection
+ *
+ * Creates a new connection, wrapping @sub_cnc which has to be a virtual connection. @sub_cnc's
+ * reference count is incremented by this call.
+ *
+ * Returns: a new #GdaConnection, or %NULL if an error occurred.
+ */
+GdaConnection *
+_gda_thread_provider_handle_virtual_connection (GdaThreadProvider *provider, GdaConnection *sub_cnc)
+{
+	ThreadConnectionData *cdata;
+	GdaServerProvider *sub_prov;
+	GdaThreadWrapper *wr;
+	GdaConnection *cnc;
+
+	g_return_val_if_fail (GDA_IS_THREAD_PROVIDER (provider), NULL);
+	g_return_val_if_fail (GDA_IS_VIRTUAL_CONNECTION (sub_cnc), NULL);
+
+	sub_prov = gda_connection_get_provider (sub_cnc);
+	g_return_val_if_fail (GDA_IS_VIRTUAL_PROVIDER (sub_prov), NULL);
+	wr = gda_thread_wrapper_new ();
+	if (!wr) {
+		g_warning (_("Multi threading is not supported or enabled"));
+		return NULL;
+	}
+
+	cnc = gda_thread_provider_create_connection (GDA_SERVER_PROVIDER (provider));
+
+	cdata = g_new0 (ThreadConnectionData, 1);
+	cdata->sub_connection = g_object_ref (sub_cnc);
+	cdata->cnc_provider = g_object_ref (sub_prov);
+	cdata->wrapper = wr;
+	cdata->handlers_ids = g_array_sized_new (FALSE, FALSE, sizeof (gulong), 2);
+	gda_connection_internal_set_provider_data (cnc, cdata, (GDestroyNotify) gda_thread_free_cnc_data);
+	setup_signals (cnc, cdata);
+
+	return cnc;
+}
+
 static void
-sub_cnc_error_cb (GdaThreadWrapper *wrapper, GdaConnection *sub_cnc, const gchar *signal,
-		  gint n_param_values, const GValue *param_values,
-		  gpointer gda_reserved, GdaConnection *wrapper_cnc)
+sub_cnc_error_cb (G_GNUC_UNUSED GdaThreadWrapper *wrapper, G_GNUC_UNUSED GdaConnection *sub_cnc,
+		  G_GNUC_UNUSED const gchar *signal,
+		  gint n_param_values, const GValue *param_values, G_GNUC_UNUSED gpointer gda_reserved,
+		  GdaConnection *wrapper_cnc)
 {
 	GdaConnectionEvent *ev;
 	g_assert (n_param_values == 1);
@@ -458,22 +543,45 @@ sub_cnc_error_cb (GdaThreadWrapper *wrapper, GdaConnection *sub_cnc, const gchar
 }
 
 static void
-sub_cnc_transaction_status_changed_cb (GdaThreadWrapper *wrapper, GdaConnection *sub_cnc, const gchar *signal,
-				       gint n_param_values, const GValue *param_values,
-				       gpointer gda_reserved, GdaConnection *wrapper_cnc)
+sub_cnc_transaction_status_changed_cb (G_GNUC_UNUSED GdaThreadWrapper *wrapper, GdaConnection *sub_cnc,
+				       G_GNUC_UNUSED const gchar *signal, G_GNUC_UNUSED gint n_param_values,
+				       G_GNUC_UNUSED const GValue *param_values,
+				       G_GNUC_UNUSED gpointer gda_reserved, GdaConnection *wrapper_cnc)
 {
 	_gda_connection_force_transaction_status (wrapper_cnc, sub_cnc);
 }
 
+static void
+sub_cnc_closed_cb (G_GNUC_UNUSED GdaThreadWrapper *wrapper, G_GNUC_UNUSED GdaConnection *sub_cnc,
+		   G_GNUC_UNUSED const gchar *signal, G_GNUC_UNUSED gint n_param_values,
+		   G_GNUC_UNUSED const GValue *param_values, G_GNUC_UNUSED gpointer gda_reserved,
+		   GdaConnection *wrapper_cnc)
+{
+	ThreadConnectionData *cdata;
+	cdata = (ThreadConnectionData*) gda_connection_internal_get_provider_data (wrapper_cnc);
+	if (!cdata) 
+		return;
+	cdata->sub_connection_has_closed = TRUE;
+}
+
 static void
 setup_signals (GdaConnection *cnc, ThreadConnectionData *cdata)
 {
 	gulong hid;
-	hid = gda_thread_wrapper_connect_raw (cdata->wrapper, cdata->sub_connection, "error",
+	hid = gda_thread_wrapper_connect_raw (cdata->wrapper, cdata->sub_connection,
+					      "conn-closed", TRUE, FALSE,
+					      (GdaThreadWrapperCallback) sub_cnc_closed_cb, cnc);
+	g_array_prepend_val (cdata->handlers_ids, hid);
+
+	hid = gda_thread_wrapper_connect_raw (cdata->wrapper, cdata->sub_connection,
+					      "error", TRUE, FALSE,
 					      (GdaThreadWrapperCallback) sub_cnc_error_cb, cnc);
 	g_array_prepend_val (cdata->handlers_ids, hid);
-	hid = gda_thread_wrapper_connect_raw (cdata->wrapper, cdata->sub_connection, "transaction-status-changed",
-					      (GdaThreadWrapperCallback) sub_cnc_transaction_status_changed_cb, cnc);
+
+	hid = gda_thread_wrapper_connect_raw (cdata->wrapper, cdata->sub_connection,
+					      "transaction-status-changed", TRUE, FALSE,
+					      (GdaThreadWrapperCallback) sub_cnc_transaction_status_changed_cb,
+					      cnc);
 	g_array_prepend_val (cdata->handlers_ids, hid);
 }
 
@@ -492,7 +600,7 @@ typedef struct {
 } CncProvData;
 
 static gpointer
-sub_thread_close_connection (CncProvData *data, GError **error)
+sub_thread_close_connection (CncProvData *data, G_GNUC_UNUSED GError **error)
 {
 	/* WARNING: function executed in sub thread! */
 	gboolean retval;
@@ -534,7 +642,7 @@ gda_thread_provider_close_connection (GdaServerProvider *provider, GdaConnection
  */
 
 static const gchar *
-sub_thread_get_server_version (CncProvData *data, GError **error)
+sub_thread_get_server_version (CncProvData *data, G_GNUC_UNUSED GError **error)
 {
 	/* WARNING: function executed in sub thread! */
 	const gchar *retval;
@@ -575,7 +683,7 @@ gda_thread_provider_get_server_version (GdaServerProvider *provider, GdaConnecti
  * Returns the database name as a string, which should be stored in @cnc's associated ThreadConnectionData structure
  */
 static const gchar *
-sub_thread_get_database (CncProvData *data, GError **error)
+sub_thread_get_database (CncProvData *data, G_GNUC_UNUSED GError **error)
 {
 	/* WARNING: function executed in sub thread! */
 	const gchar *retval;
@@ -630,7 +738,7 @@ typedef struct {
 } SupportsOperationData;
 
 static gpointer
-sub_thread_supports_operation (SupportsOperationData *data, GError **error)
+sub_thread_supports_operation (SupportsOperationData *data, G_GNUC_UNUSED GError **error)
 {
 	/* WARNING: function executed in sub thread! */
 	gboolean retval;
@@ -643,7 +751,8 @@ sub_thread_supports_operation (SupportsOperationData *data, GError **error)
 
 static gboolean
 gda_thread_provider_supports_operation (GdaServerProvider *provider, GdaConnection *cnc,
-					GdaServerOperationType type, GdaSet *options)
+					G_GNUC_UNUSED GdaServerOperationType type,
+					G_GNUC_UNUSED GdaSet *options)
 {
 	ThreadConnectionData *cdata;
 	SupportsOperationData wdata;
@@ -800,8 +909,9 @@ sub_thread_perform_operation (PerformOperationData *data, GError **error)
 
 static gboolean
 gda_thread_provider_perform_operation (GdaServerProvider *provider, GdaConnection *cnc,
-				       GdaServerOperation *op, guint *task_id, 
-				       GdaServerProviderAsyncCallback async_cb, gpointer cb_data, GError **error)
+				       GdaServerOperation *op, G_GNUC_UNUSED guint *task_id,
+				       GdaServerProviderAsyncCallback async_cb, G_GNUC_UNUSED gpointer cb_data,
+				       GError **error)
 {
 	ThreadConnectionData *cdata;
 	PerformOperationData wdata;
@@ -943,7 +1053,7 @@ gda_thread_provider_commit_transaction (GdaServerProvider *provider, GdaConnecti
 
 	jid = gda_thread_wrapper_execute (cdata->wrapper, 
 					  (GdaThreadWrapperFunc) sub_thread_commit_transaction, &wdata, NULL, error);
-	res = gda_thread_wrapper_fetch_result (cdata->wrapper, TRUE, jid, NULL);
+	res = gda_thread_wrapper_fetch_result (cdata->wrapper, TRUE, jid, error);
 	return GPOINTER_TO_INT (res) ? TRUE : FALSE;
 }
 
@@ -990,7 +1100,7 @@ gda_thread_provider_rollback_transaction (GdaServerProvider *provider, GdaConnec
 
 	jid = gda_thread_wrapper_execute (cdata->wrapper, 
 					  (GdaThreadWrapperFunc) sub_thread_rollback_transaction, &wdata, NULL, error);
-	res = gda_thread_wrapper_fetch_result (cdata->wrapper, TRUE, jid, NULL);
+	res = gda_thread_wrapper_fetch_result (cdata->wrapper, TRUE, jid, error);
 	return GPOINTER_TO_INT (res) ? TRUE : FALSE;
 }
 
@@ -1177,7 +1287,7 @@ typedef struct {
 } GetDataHandlerData;
 
 static GdaDataHandler *
-sub_thread_get_data_handler (GetDataHandlerData *data, GError **error)
+sub_thread_get_data_handler (GetDataHandlerData *data, G_GNUC_UNUSED GError **error)
 {
 	/* WARNING: function executed in sub thread! */
 	GdaDataHandler *retval;
@@ -1227,7 +1337,7 @@ typedef struct {
 } GetDefaultDbmsTypeData;
 
 static const gchar *
-sub_thread_get_default_dbms_type (GetDefaultDbmsTypeData *data, GError **error)
+sub_thread_get_default_dbms_type (GetDefaultDbmsTypeData *data, G_GNUC_UNUSED GError **error)
 {
 	/* WARNING: function executed in sub thread! */
 	const gchar *retval;
@@ -1271,11 +1381,12 @@ gda_thread_provider_get_default_dbms_type (GdaServerProvider *provider, GdaConne
  */
 
 static GdaSqlParser *
-sub_thread_create_parser (CncProvData *data, GError **error)
+sub_thread_create_parser (CncProvData *data, G_GNUC_UNUSED GError **error)
 {
 	/* WARNING: function executed in sub thread! */
-	GdaSqlParser *parser;
-	parser = PROV_CLASS (data->prov)->create_parser (data->prov, data->cnc);
+	GdaSqlParser *parser = NULL;
+	if (PROV_CLASS (data->prov)->create_parser)
+		parser = PROV_CLASS (data->prov)->create_parser (data->prov, data->cnc);
 #ifdef GDA_DEBUG_NO
 	g_print ("/%s() => %p\n", __FUNCTION__, parser);
 #endif
@@ -1325,13 +1436,18 @@ typedef struct {
 	GSList **params_used;
 } StmtToSqlData;
 
-static const gchar *
+static gchar *
 sub_thread_statement_to_sql (StmtToSqlData *data, GError **error)
 {
 	/* WARNING: function executed in sub thread! */
-	const gchar *retval;
-	retval = PROV_CLASS (data->prov)->statement_to_sql (data->prov, data->cnc, data->stmt,
-							    data->params, data->flags, data->params_used, error);
+	gchar *retval;
+	if (PROV_CLASS (data->prov)->statement_to_sql)
+		retval = PROV_CLASS (data->prov)->statement_to_sql (data->prov, data->cnc, data->stmt,
+								    data->params, data->flags, data->params_used, error);
+	else
+		retval = gda_statement_to_sql_extended (data->stmt, data->cnc, data->params, data->flags, data->params_used,
+							error);
+		
 #ifdef GDA_DEBUG_NO
 	g_print ("/%s() => %s\n", __FUNCTION__, retval);
 #endif
@@ -1341,7 +1457,7 @@ sub_thread_statement_to_sql (StmtToSqlData *data, GError **error)
 static gchar *
 gda_thread_provider_statement_to_sql (GdaServerProvider *provider, GdaConnection *cnc,
 				      GdaStatement *stmt, GdaSet *params, GdaStatementSqlFlag flags,
-				      GSList **params_used, GError **error)
+				      GSList **params_used, G_GNUC_UNUSED GError **error)
 {
 	ThreadConnectionData *cdata;
 	StmtToSqlData wdata;
@@ -1356,7 +1472,7 @@ gda_thread_provider_statement_to_sql (GdaServerProvider *provider, GdaConnection
 	cdata = (ThreadConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
 		return NULL;
-	
+
 	wdata.prov = cdata->cnc_provider;
 	wdata.cnc = cdata->sub_connection;
 	wdata.stmt = stmt;
@@ -1456,6 +1572,13 @@ sub_thread_execute_statement (ExecuteStatementData *data, GError **error)
 {
 	/* WARNING: function executed in sub thread! */
 	GObject *retval;
+	
+#ifdef GDA_DEBUG_NO
+	g_print ("%p Starting sleep in %s()...\n", g_thread_self (), __FUNCTION__);
+	sleep (10);
+	g_print ("%p finished sleeping in %s().\n", g_thread_self (), __FUNCTION__);
+#endif
+
 	retval = PROV_CLASS (data->prov)->statement_execute (data->prov,
 							     data->cnc,
 							     data->stmt,
@@ -1496,7 +1619,13 @@ gda_thread_provider_statement_execute (GdaServerProvider *provider, GdaConnectio
 
 	cdata = (ThreadConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
-		return FALSE;
+		return NULL;
+
+	if (check_cnc_closed (cnc, cdata)) {
+		g_set_error (error, GDA_CONNECTION_ERROR, GDA_CONNECTION_CLOSED_ERROR,
+			     _("Connection is closed"));
+		return NULL;
+	}
 	
 	if (async_cb) {
 		ExecuteStatementData *wdata;
@@ -1550,7 +1679,8 @@ gda_thread_provider_statement_execute (GdaServerProvider *provider, GdaConnectio
 }
 
 static gboolean
-gda_thread_provider_handle_async (GdaServerProvider *provider, GdaConnection *cnc, GError **error)
+gda_thread_provider_handle_async (GdaServerProvider *provider, GdaConnection *cnc,
+				  G_GNUC_UNUSED GError **error)
 {
 	ThreadConnectionData *cdata;
 	GObject *res;
@@ -1564,16 +1694,20 @@ gda_thread_provider_handle_async (GdaServerProvider *provider, GdaConnection *cn
 	if (!cdata) 
 		return FALSE;
 
+	if (check_cnc_closed (cnc, cdata))
+		return TRUE;
+
 	if (!cdata->async_tasks)
 		return TRUE;
 
 	atd = (ThreadConnectionAsyncTask*) cdata->async_tasks->data;
 	res = (GObject*) gda_thread_wrapper_fetch_result (cdata->wrapper, FALSE, atd->jid, &lerror);
-	if (res) {
+	if (res || lerror) {
 		atd->async_cb (provider, cnc, atd->jid, res, lerror, atd->cb_data);
 		if (lerror)
 			g_error_free (lerror);
-		g_object_unref (res);
+		if (res)
+			g_object_unref (res);
 
 		_ThreadConnectionAsyncTask_free (atd);
 		cdata->async_tasks = g_slist_delete_link (cdata->async_tasks, cdata->async_tasks);
@@ -1850,7 +1984,7 @@ sub_thread_xa_recover (CncProvData *data, GError **error)
 #ifdef GDA_DEBUG_NO
 	g_print ("/%s() => %p\n", __FUNCTION__, retval);
 #endif
-	return GINT_TO_POINTER (retval ? 1 : 0);
+	return retval;
 }
 
 static GList *
@@ -1883,6 +2017,72 @@ gda_thread_provider_xa_recover (GdaServerProvider *provider, GdaConnection *cnc,
 	return res;
 }
 
+/*
+ * Quote SQL identifier
+ *
+ * Returns: a new string
+ */
+typedef struct {
+	GdaServerProvider *prov;
+	GdaConnection *cnc;
+	const gchar *id;
+	gboolean for_meta_store;
+	gboolean force_quotes;
+} SqlIdentifierQuoteData;
+
+static gchar *
+sub_thread_identifier_quote (SqlIdentifierQuoteData *data, G_GNUC_UNUSED GError **error)
+{
+	/* WARNING: function executed in sub thread! */
+	gchar *retval;
+	retval = gda_sql_identifier_quote (data->id, data->cnc, data->prov,
+					   data->for_meta_store, data->force_quotes);
+#ifdef GDA_DEBUG_NO
+	g_print ("/%s() => %p\n", __FUNCTION__, retval);
+#endif
+	return retval;
+}
+
+static gchar *
+gda_thread_provider_identifier_quote (GdaServerProvider *provider, GdaConnection *cnc,
+				      const gchar *id,
+				      gboolean for_meta_store, gboolean force_quotes)
+{
+	ThreadConnectionData *cdata;
+	SqlIdentifierQuoteData wdata;
+	gchar *res;
+	guint jid;
+
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
+
+	cdata = (ThreadConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	if (!cdata) 
+		return NULL;
+	
+	wdata.prov = cdata->cnc_provider;
+	wdata.cnc = cdata->sub_connection;
+	wdata.id = id;
+	wdata.for_meta_store = for_meta_store;
+	wdata.force_quotes = force_quotes;
+
+	jid = gda_thread_wrapper_execute (cdata->wrapper, 
+					  (GdaThreadWrapperFunc) sub_thread_identifier_quote, &wdata, NULL, NULL);
+	res = gda_thread_wrapper_fetch_result (cdata->wrapper, TRUE, jid, NULL);
+	return res;
+}
+
+static gpointer
+sub_thread_unref_connection (GdaConnection *cnc, G_GNUC_UNUSED GError **error)
+{
+	/* WARNING: function executed in sub thread! */
+	g_object_unref (cnc);
+#ifdef GDA_DEBUG_NO
+	g_print ("/%s()\n", __FUNCTION__);
+#endif
+	return NULL;
+}
+
 /*
  * Free connection's specific data
  */
@@ -1893,16 +2093,18 @@ gda_thread_free_cnc_data (ThreadConnectionData *cdata)
 		return;
 
 	/* disconnect signals handlers */
-	gint i;
+	gsize i;
 	for (i = 0; i < cdata->handlers_ids->len; i++) {
 		gulong hid = g_array_index (cdata->handlers_ids, gulong, i);
 		gda_thread_wrapper_disconnect (cdata->wrapper, hid);
 	}
 
 	/* unref cdata->sub_connection in sub thread */
-	gda_thread_wrapper_execute_void (cdata->wrapper, 
-					 (GdaThreadWrapperVoidFunc) g_object_unref,
-					 cdata->sub_connection, NULL, NULL);
+	guint jid;
+	jid = gda_thread_wrapper_execute (cdata->wrapper, 
+					  (GdaThreadWrapperFunc) sub_thread_unref_connection,
+					  cdata->sub_connection, NULL, NULL);
+	gda_thread_wrapper_fetch_result (cdata->wrapper, TRUE, jid, NULL);
 	g_object_unref (cdata->wrapper);
 
 	/* free async data */
@@ -1910,5 +2112,8 @@ gda_thread_free_cnc_data (ThreadConnectionData *cdata)
 		g_slist_foreach (cdata->async_tasks, (GFunc) _ThreadConnectionAsyncTask_free, NULL);
 		g_slist_free (cdata->async_tasks);
 	}
+
+	g_object_unref (cdata->cnc_provider);
+
 	g_free (cdata);
 }
