@@ -87,7 +87,8 @@ browser_canvas_get_type (void)
 			NULL,
 			sizeof (BrowserCanvas),
 			0,
-			(GInstanceInitFunc) browser_canvas_init
+			(GInstanceInitFunc) browser_canvas_init,
+			0
 		};		
 
 		type = g_type_register_static (GTK_TYPE_SCROLLED_WINDOW, "BrowserCanvas", &info, 0);
@@ -186,7 +187,8 @@ idle_add_canvas_cb (BrowserCanvas *canvas)
 }
 
 static void
-drag_begin_cb (BrowserCanvas *canvas, GdkDragContext *drag_context, GooCanvas *gcanvas)
+drag_begin_cb (BrowserCanvas *canvas, G_GNUC_UNUSED GdkDragContext *drag_context,
+	       G_GNUC_UNUSED GooCanvas *gcanvas)
 {
 	BrowserCanvasItem *citem;
 
@@ -202,7 +204,7 @@ drag_begin_cb (BrowserCanvas *canvas, GdkDragContext *drag_context, GooCanvas *g
 static void
 drag_data_get_cb (BrowserCanvas *canvas, GdkDragContext *drag_context,
 		  GtkSelectionData *data, guint info,
-		  guint time, GooCanvas *gcanvas)
+		  guint time, G_GNUC_UNUSED GooCanvas *gcanvas)
 {
 	BrowserCanvasItem *citem;
 
@@ -215,14 +217,14 @@ drag_data_get_cb (BrowserCanvas *canvas, GdkDragContext *drag_context,
 }
 
 static void
-drag_data_received_cb (BrowserCanvas *canvas, GdkDragContext *context,
-		       gint x, gint y, GtkSelectionData *data,
-		       guint info, guint time, GooCanvas *gcanvas)
+drag_data_received_cb (G_GNUC_UNUSED BrowserCanvas *canvas, GdkDragContext *context,
+		       gint x, gint y, G_GNUC_UNUSED GtkSelectionData *data,
+		       G_GNUC_UNUSED guint info, guint time, GooCanvas *gcanvas)
 {
 	GooCanvasItem *item;
 	item = goo_canvas_get_item_at (gcanvas, x, y, TRUE);
 	if (item) {
-		g_print ("Dragged into %s\n", G_OBJECT_TYPE_NAME (item));
+		/*g_print ("Dragged into %s\n", G_OBJECT_TYPE_NAME (item));*/
 		gtk_drag_finish (context, TRUE, FALSE, time);
 	}
 	else {
@@ -231,7 +233,7 @@ drag_data_received_cb (BrowserCanvas *canvas, GdkDragContext *context,
 }
 
 static gboolean
-canvas_scroll_event_cb (GooCanvas *gcanvas, GdkEvent *event, BrowserCanvas *canvas)
+canvas_scroll_event_cb (G_GNUC_UNUSED GooCanvas *gcanvas, GdkEvent *event, BrowserCanvas *canvas)
 {
 	gboolean done = TRUE;
 	GdkEventScroll *ev = (GdkEventScroll *) event;
@@ -262,7 +264,7 @@ canvas_scroll_event_cb (GooCanvas *gcanvas, GdkEvent *event, BrowserCanvas *canv
 static GdkCursor *hand_cursor = NULL;
 
 static gboolean
-motion_notify_event_cb (BrowserCanvas *canvas, GdkEvent *event, GooCanvas *gcanvas)
+motion_notify_event_cb (BrowserCanvas *canvas, GdkEvent *event, G_GNUC_UNUSED GooCanvas *gcanvas)
 {
 	gboolean done = TRUE;
 
@@ -272,17 +274,36 @@ motion_notify_event_cb (BrowserCanvas *canvas, GdkEvent *event, GooCanvas *gcanv
 			if (canvas->priv->canvas_moving) {
 				GtkAdjustment *ha, *va;
 				gdouble x, y;
+				gdouble upper, lower, page_size;
 				ha = gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (canvas));
 				va = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (canvas));
 
+#if GTK_CHECK_VERSION(2,18,0)
+				upper = gtk_adjustment_get_upper (ha);
+				lower = gtk_adjustment_get_lower (ha);
+				page_size = gtk_adjustment_get_page_size (ha);
+#else
+				upper = ha->upper;
+				lower = ha->lower;
+				page_size = ha->page_size;
+#endif
 				x = gtk_adjustment_get_value (ha);
 				x = CLAMP (x + canvas->xmouse - ((GdkEventMotion*) event)->x,
-					   ha->lower, ha->upper - ha->page_size);
+					   lower, upper - page_size);
 				gtk_adjustment_set_value (ha, x);
 
+#if GTK_CHECK_VERSION(2,18,0)
+				upper = gtk_adjustment_get_upper (va);
+				lower = gtk_adjustment_get_lower (va);
+				page_size = gtk_adjustment_get_page_size (va);
+#else
+				upper = va->upper;
+				lower = va->lower;
+				page_size = va->page_size;
+#endif
 				y = gtk_adjustment_get_value (va);
 				y = CLAMP (y + canvas->ymouse - ((GdkEventMotion*) event)->y,
-					   va->lower, va->upper - va->page_size);
+					   lower, upper - page_size);
 				gtk_adjustment_set_value (va, y);
 			}
 			else {
@@ -291,7 +312,13 @@ motion_notify_event_cb (BrowserCanvas *canvas, GdkEvent *event, GooCanvas *gcanv
 				canvas->priv->canvas_moving = TRUE;
 				if (! hand_cursor)
 					hand_cursor = gdk_cursor_new (GDK_HAND2);
-				gdk_window_set_cursor (GTK_WIDGET (canvas)->window, hand_cursor);
+#if GTK_CHECK_VERSION(2,18,0)
+				gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (canvas)),
+						       hand_cursor);
+#else
+				gdk_window_set_cursor (GTK_WIDGET (canvas)->window,
+						       hand_cursor);
+#endif
 			}
 		}
 		done = TRUE;
@@ -341,50 +368,50 @@ canvas_event_cb (BrowserCanvas *canvas, GdkEvent *event, GooCanvas *gcanvas)
 				else {
 					mitem = gtk_separator_menu_item_new ();
 					gtk_widget_show (mitem);
-					gtk_menu_append (menu, mitem);
+					gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 				}
 				mitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_ZOOM_IN, NULL);
 				gtk_widget_show (mitem);
-				gtk_menu_append (menu, mitem);
+				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 				g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (popup_zoom_in_cb), canvas);
 				mitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_ZOOM_OUT, NULL);
 				gtk_widget_show (mitem);
-				gtk_menu_append (menu, mitem);
+				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 				g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (popup_zoom_out_cb), canvas);
 				mitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_ZOOM_FIT, NULL);
 				gtk_widget_show (mitem);
-				gtk_menu_append (menu, mitem);
+				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 				g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (popup_zoom_fit_cb), canvas);
 
 				mitem = gtk_separator_menu_item_new ();
 				gtk_widget_show (mitem);
-				gtk_menu_append (menu, mitem);
+				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 
 #ifdef HAVE_GRAPHVIZ
 				mitem = gtk_menu_item_new_with_label (_("Linear layout"));
 				gtk_widget_show (mitem);
-				gtk_menu_append (menu, mitem);
+				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 				g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (popup_layout_default_cb), canvas);
 
 				mitem = gtk_menu_item_new_with_label (_("Radial layout"));
 				gtk_widget_show (mitem);
-				gtk_menu_append (menu, mitem);
+				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 				g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (popup_layout_radial_cb), canvas);
 
 				mitem = gtk_separator_menu_item_new ();
 				gtk_widget_show (mitem);
-				gtk_menu_append (menu, mitem);
+				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 #endif
 				
 				mitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_SAVE_AS, NULL);
 				gtk_widget_show (mitem);
-				gtk_menu_append (menu, mitem);
+				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 				g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (popup_export_cb), canvas);
 
 				mitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_PRINT, NULL);
 				gtk_widget_show (mitem);
 				g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (popup_print_cb), canvas);
-				gtk_menu_append (menu, mitem);
+				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 
 				gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
 						NULL, NULL, ((GdkEventButton *)event)->button,
@@ -396,7 +423,11 @@ canvas_event_cb (BrowserCanvas *canvas, GdkEvent *event, GooCanvas *gcanvas)
 	case GDK_BUTTON_RELEASE:
 		if (canvas->priv->canvas_moving) {
 			canvas->priv->canvas_moving = FALSE;
+#if GTK_CHECK_VERSION(2,18,0)
+			gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (canvas)), NULL);
+#else
 			gdk_window_set_cursor (GTK_WIDGET (canvas)->window, NULL);
+#endif
 		}
 		break;
 	case GDK_2BUTTON_PRESS:
@@ -429,37 +460,37 @@ canvas_event_cb (BrowserCanvas *canvas, GdkEvent *event, GooCanvas *gcanvas)
 }
 
 static void
-popup_layout_default_cb (GtkMenuItem *mitem, BrowserCanvas *canvas)
+popup_layout_default_cb (G_GNUC_UNUSED GtkMenuItem *mitem, BrowserCanvas *canvas)
 {
 	browser_canvas_perform_auto_layout (canvas, TRUE, BROWSER_CANVAS_LAYOUT_DEFAULT);
 }
 
 static void
-popup_layout_radial_cb (GtkMenuItem *mitem, BrowserCanvas *canvas)
+popup_layout_radial_cb (G_GNUC_UNUSED GtkMenuItem *mitem, BrowserCanvas *canvas)
 {
 	browser_canvas_perform_auto_layout (canvas, TRUE, BROWSER_CANVAS_LAYOUT_RADIAL);
 }
 
 static void
-popup_zoom_in_cb (GtkMenuItem *mitem, BrowserCanvas *canvas)
+popup_zoom_in_cb (G_GNUC_UNUSED GtkMenuItem *mitem, BrowserCanvas *canvas)
 {
 	browser_canvas_set_zoom_factor (canvas, browser_canvas_get_zoom_factor (canvas) + .05);
 }
 
 static void
-popup_zoom_out_cb (GtkMenuItem *mitem, BrowserCanvas *canvas)
+popup_zoom_out_cb (G_GNUC_UNUSED GtkMenuItem *mitem, BrowserCanvas *canvas)
 {
 	browser_canvas_set_zoom_factor (canvas, browser_canvas_get_zoom_factor (canvas) - .05);
 }
 
 static void
-popup_zoom_fit_cb (GtkMenuItem *mitem, BrowserCanvas *canvas)
+popup_zoom_fit_cb (G_GNUC_UNUSED GtkMenuItem *mitem, BrowserCanvas *canvas)
 {
 	browser_canvas_fit_zoom_factor (canvas);
 }
 
 static void
-popup_export_cb (GtkMenuItem *mitem, BrowserCanvas *canvas)
+popup_export_cb (G_GNUC_UNUSED GtkMenuItem *mitem, BrowserCanvas *canvas)
 {
 	GtkWidget *dlg;
 	gint result;
@@ -468,8 +499,13 @@ popup_export_cb (GtkMenuItem *mitem, BrowserCanvas *canvas)
 
 #define MARGIN 5.
 
+#if GTK_CHECK_VERSION(2,18,0)
+	if (!gtk_widget_is_toplevel (toplevel))
+		toplevel = NULL;
+#else
 	if (!GTK_WIDGET_TOPLEVEL (toplevel))
 		toplevel = NULL;
+#endif
 
 	dlg = gtk_file_chooser_dialog_new (_("Save diagram as"), (GtkWindow*) toplevel,
 					   GTK_FILE_CHOOSER_ACTION_SAVE, 
@@ -561,9 +597,10 @@ popup_export_cb (GtkMenuItem *mitem, BrowserCanvas *canvas)
 			if (error) {
 				GtkWidget *errdlg;
 
-				errdlg = gtk_message_dialog_new_with_markup ((GtkWindow*) toplevel,
-									     GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, 
-									     GTK_BUTTONS_CLOSE, "%s", error);
+				errdlg = gtk_message_dialog_new ((GtkWindow*) toplevel,
+								 GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, 
+								 GTK_BUTTONS_CLOSE, NULL);
+				gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (errdlg), error);
 				g_free (error);
 				gtk_dialog_run (GTK_DIALOG (errdlg));
 				gtk_widget_destroy (errdlg);
@@ -577,7 +614,7 @@ popup_export_cb (GtkMenuItem *mitem, BrowserCanvas *canvas)
 }
 
 static void
-popup_print_cb (GtkMenuItem *mitem, BrowserCanvas *canvas)
+popup_print_cb (G_GNUC_UNUSED GtkMenuItem *mitem, BrowserCanvas *canvas)
 {
 	browser_canvas_print (canvas);
 }
@@ -662,7 +699,7 @@ browser_canvas_finalize (GObject *object)
 static void 
 browser_canvas_set_property (GObject *object,
 			   guint param_id,
-			   const GValue *value,
+			   G_GNUC_UNUSED const GValue *value,
 			   GParamSpec *pspec)
 {
 	BrowserCanvas *canvas;
@@ -670,13 +707,16 @@ browser_canvas_set_property (GObject *object,
 	canvas = BROWSER_CANVAS (object);
 
 	switch (param_id) {
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+		break;
 	}
 }
 
 static void
 browser_canvas_get_property (GObject *object,
 			   guint param_id,
-			   GValue *value,
+			   G_GNUC_UNUSED GValue *value,
 			   GParamSpec *pspec)
 {
 	BrowserCanvas *canvas;
@@ -684,6 +724,9 @@ browser_canvas_get_property (GObject *object,
 	canvas = BROWSER_CANVAS (object);
 
 	switch (param_id) {
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+		break;
 	}
 }
 
@@ -744,8 +787,15 @@ browser_canvas_fit_zoom_factor (BrowserCanvas *canvas)
 	g_return_val_if_fail (IS_BROWSER_CANVAS (canvas), 1.);
 	g_return_val_if_fail (canvas->priv, 1.);
 
+#if GTK_CHECK_VERSION(2,18,0)
+	GtkAllocation alloc;
+	gtk_widget_get_allocation (GTK_WIDGET (canvas), &alloc);
+	xall = alloc.width;
+	yall = alloc.height;
+#else
 	xall = GTK_WIDGET (canvas)->allocation.width;
 	yall = GTK_WIDGET (canvas)->allocation.height;
+#endif
 
 	goo_canvas_item_get_bounds (GOO_CANVAS_ITEM (goo_canvas_get_root_item (canvas->priv->goocanvas)),
 				    &bounds);
@@ -872,7 +922,7 @@ browser_canvas_perform_auto_layout (BrowserCanvas *canvas, gboolean animate, Bro
 
 #ifndef HAVE_GRAPHVIZ
 	g_message ("GraphViz library support not compiled, cannot do graph layout...\n");
-	return FALSE;
+	return;
 #else
 	BrowserCanvasClass *class = BROWSER_CANVAS_CLASS (G_OBJECT_GET_CLASS (canvas));
 	GSList *list, *layout_items;
@@ -977,11 +1027,10 @@ browser_canvas_perform_auto_layout (BrowserCanvas *canvas, gboolean animate, Bro
 		gvLayout (gvc, graph, "circo");
 		break;
 	}
-        gvRender (gvc, graph, "dot", NULL);
+        /*gvRender (gvc, graph, "dot", NULL);*/
 	/*gvRenderFilename (gvc, graph, "png", "out.png");*/
         /*gvRender (gvc, graph, "dot", stdout);*/
 
-
 	for (list = nodes_list; list; list = list->next) {
 		NodeLayout *nl = (NodeLayout*) list->data;
 		nl->end_x = ND_coord_i (nl->node).x - (nl->width / 2.);
@@ -1019,7 +1068,7 @@ browser_canvas_perform_auto_layout (BrowserCanvas *canvas, gboolean animate, Bro
 
 #ifdef HAVE_GRAPHVIZ
 static gdouble
-compute_animation_inc (float start, float stop, float current)
+compute_animation_inc (float start, float stop, G_GNUC_UNUSED float current)
 {
         gdouble inc;
 #ifndef PI
@@ -1189,7 +1238,7 @@ browser_canvas_item_toggle_select (BrowserCanvas *canvas, BrowserCanvasItem *ite
 }
 
 void
-browser_canvas_translate_item (BrowserCanvas *canvas, BrowserCanvasItem *item,
+browser_canvas_translate_item (G_GNUC_UNUSED BrowserCanvas *canvas, BrowserCanvasItem *item,
 			       gdouble dx, gdouble dy)
 {
 	browser_canvas_item_translate (item, dx, dy);
