@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 The GNOME Foundation
+ * Copyright (C) 2009 - 2010 The GNOME Foundation
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -49,12 +49,6 @@ static void table_columns_dispose   (GObject *object);
 
 static void meta_changed_cb (BrowserConnection *bcnc, GdaMetaStruct *mstruct, TableColumns *tcolumns);
 
-enum {
-	SELECTION_CHANGED,
-	LAST_SIGNAL
-};
-
-static guint table_columns_signals[LAST_SIGNAL] = { 0 };
 static GObjectClass *parent_class = NULL;
 
 
@@ -74,7 +68,7 @@ table_columns_class_init (TableColumnsClass *klass)
 
 
 static void
-table_columns_init (TableColumns *tcolumns, TableColumnsClass *klass)
+table_columns_init (TableColumns *tcolumns, G_GNUC_UNUSED TableColumnsClass *klass)
 {
 	tcolumns->priv = g_new0 (TableColumnsPrivate, 1);
 	tcolumns->priv->idle_update_columns = 0;
@@ -119,7 +113,8 @@ table_columns_get_type (void)
 			NULL,
 			sizeof (TableColumns),
 			0,
-			(GInstanceInitFunc) table_columns_init
+			(GInstanceInitFunc) table_columns_init,
+			0
 		};
 		type = g_type_register_static (GTK_TYPE_VBOX, "TableColumns", &columns, 0);
 	}
@@ -153,7 +148,7 @@ static gboolean motion_notify_event (GtkWidget *text_view, GdkEventMotion *event
 static gboolean visibility_notify_event (GtkWidget *text_view, GdkEventVisibility *event, TableColumns *tcolumns);
 
 static void
-meta_changed_cb (BrowserConnection *bcnc, GdaMetaStruct *mstruct, TableColumns *tcolumns)
+meta_changed_cb (G_GNUC_UNUSED BrowserConnection *bcnc, GdaMetaStruct *mstruct, TableColumns *tcolumns)
 {
 	GtkTextBuffer *tbuffer;
 	GtkTextIter start, end;
@@ -301,6 +296,9 @@ meta_changed_cb (BrowserConnection *bcnc, GdaMetaStruct *mstruct, TableColumns *
 						cvalue = gda_data_model_get_value_at (model, 0, i, NULL);
 						if (!current_value || 
 						    gda_value_compare (cvalue, current_value)) {
+							if (current_value)
+								gtk_text_buffer_insert (tbuffer, &current,
+											"\n\n", -1);
 							gtk_text_buffer_insert_with_tags_by_name (tbuffer,
 												  &current, 
 												  _("Unique constraint"), -1,
@@ -314,6 +312,59 @@ meta_changed_cb (BrowserConnection *bcnc, GdaMetaStruct *mstruct, TableColumns *
 						gtk_text_buffer_insert (tbuffer, &current, 
 									g_value_get_string (cvalue), -1);
 					}
+					gtk_text_buffer_insert (tbuffer, &current, "\n\n", -1);
+				}
+				g_object_unref (model);
+			}
+
+			/* reverse FK constraints */
+			g_value_set_string ((catalog_v = gda_value_new (G_TYPE_STRING)), dbo->obj_catalog);
+			model = gda_meta_store_extract (browser_connection_get_meta_store (tcolumns->priv->bcnc),
+							"SELECT c.table_schema, c.table_name, t.table_short_name FROM _referential_constraints c NATURAL JOIN _tables t WHERE ref_table_catalog = ##catalog::string AND ref_table_schema=##schema::string AND ref_table_name=##tname::string", &error,
+							"catalog", catalog_v,
+							"schema", schema_v,
+							"tname", name_v, NULL);
+			if (model) {
+				gint nrows;
+				
+				/*gda_data_model_dump (model, NULL);*/
+				nrows = gda_data_model_get_n_rows (model);
+				if (nrows > 0) {
+					gtk_text_buffer_insert_with_tags_by_name (tbuffer,
+										  &current, 
+										  _("Tables referencing this one"), -1,
+										  "section", NULL);
+					gtk_text_buffer_insert (tbuffer, &current, "\n", -1);
+
+					gint i;
+					const GValue *cvalue[3];
+					for (i = 0; i < nrows; i++) {
+						if (i > 0)
+							gtk_text_buffer_insert (tbuffer, &current, "\n", -1);
+
+						gint j;
+						for (j = 0; j < 3; j++) {
+							cvalue[j] = gda_data_model_get_value_at (model, j, i, NULL);
+							if (! cvalue[j])
+								break;
+						}
+						if (j != 3)
+							break;
+						GtkTextTag *tag;
+						tag = gtk_text_buffer_create_tag (tbuffer, NULL, 
+										  "foreground", "blue", 
+										  "weight", PANGO_WEIGHT_NORMAL,
+										  "underline", PANGO_UNDERLINE_SINGLE, 
+										  NULL);
+						g_object_set_data_full (G_OBJECT (tag), "table_schema", 
+									g_value_dup_string (cvalue[0]), g_free);
+						g_object_set_data_full (G_OBJECT (tag), "table_name", 
+									g_value_dup_string (cvalue[1]), g_free);
+						g_object_set_data_full (G_OBJECT (tag), "table_short_name", 
+									g_value_dup_string (cvalue[2]), g_free);
+						gtk_text_buffer_insert_with_tags (tbuffer, &current,
+										  g_value_get_string (cvalue[2]), -1, tag, NULL);
+					}
 				}
 				g_object_unref (model);
 			}
@@ -343,7 +394,8 @@ table_columns_new (TableInfo *tinfo)
 		COLUMN_TYPE,
 		COLUMN_NOTNULL,
 		COLUMN_DEFAULT,
-		COLUMN_ICON
+		COLUMN_ICON,
+		COLUMN_DETAILS
 	};
 
 	g_return_val_if_fail (IS_TABLE_INFO (tinfo), NULL);
@@ -382,13 +434,14 @@ table_columns_new (TableInfo *tinfo)
         GtkCellRenderer *renderer;
         GtkTreeViewColumn *column;
 
-        model = gdaui_tree_store_new (tcolumns->priv->columns_tree, 5,
+        model = gdaui_tree_store_new (tcolumns->priv->columns_tree, 6,
                                       G_TYPE_STRING, MGR_COLUMNS_COL_NAME_ATT_NAME,
 				      G_TYPE_STRING, MGR_COLUMNS_COL_TYPE_ATT_NAME,
 				      G_TYPE_BOOLEAN, MGR_COLUMNS_COL_NOTNULL_ATT_NAME,
 				      G_TYPE_STRING, MGR_COLUMNS_COL_DEFAULT_ATT_NAME,
-                                      G_TYPE_OBJECT, "icon");
-        treeview = gtk_tree_view_new_with_model (model);
+                                      G_TYPE_OBJECT, "icon",
+				      G_TYPE_STRING, MGR_COLUMNS_COL_DETAILS);
+        treeview = browser_make_tree_view (model);
         g_object_unref (model);
 
         /* Colum: Name */
@@ -423,6 +476,12 @@ table_columns_new (TableInfo *tinfo)
 							   "text", COLUMN_DEFAULT, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
+	/* Colum: Details */
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes ("Details", renderer,
+							   "text", COLUMN_DETAILS, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
+
 	/* scrolled window packing */
         GtkWidget *sw;
         sw = gtk_scrolled_window_new (NULL, NULL);
@@ -548,11 +607,16 @@ set_cursor_if_appropriate (GtkTextView *text_view, gint x, gint y, TableColumns
  * (e.g. when a window covering it got iconified).
  */
 static gboolean
-visibility_notify_event (GtkWidget *text_view, GdkEventVisibility *event, TableColumns *tcolumns)
+visibility_notify_event (GtkWidget *text_view, G_GNUC_UNUSED GdkEventVisibility *event,
+			 TableColumns *tcolumns)
 {
 	gint wx, wy, bx, by;
 	
+#if GTK_CHECK_VERSION(2,18,0)
+	gdk_window_get_pointer (gtk_widget_get_window (text_view), &wx, &wy, NULL);
+#else
 	gdk_window_get_pointer (text_view->window, &wx, &wy, NULL);
+#endif
 	
 	gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view), 
 					       GTK_TEXT_WINDOW_WIDGET,
@@ -576,8 +640,13 @@ motion_notify_event (GtkWidget *text_view, GdkEventMotion *event, TableColumns *
 					       event->x, event->y, &x, &y);
 	
 	set_cursor_if_appropriate (GTK_TEXT_VIEW (text_view), x, y, tcolumns);
-	
+
+#if GTK_CHECK_VERSION(2,18,0)
+	gdk_window_get_pointer (gtk_widget_get_window (text_view), NULL, NULL, NULL);
+#else
 	gdk_window_get_pointer (text_view->window, NULL, NULL, NULL);
+#endif
+
 	return FALSE;
 }
 
@@ -586,7 +655,7 @@ motion_notify_event (GtkWidget *text_view, GdkEventMotion *event, TableColumns *
  * by the data attached to it.
  */
 static void
-follow_if_link (GtkWidget *text_view, GtkTextIter *iter, TableColumns *tcolumns)
+follow_if_link (G_GNUC_UNUSED GtkWidget *text_view, GtkTextIter *iter, TableColumns *tcolumns)
 {
 	GSList *tags = NULL, *tagp = NULL;
 	
