@@ -5,17 +5,20 @@
 #include <libgda/libgda.h>
 #include <libgda/binreloc/gda-binreloc.h>
 #include <glib/gi18n-lib.h>
+#include "gda-tree-mgr-xml.h"
 
 gchar *prov = NULL;
 gchar *op = NULL;
 gboolean list_ops = FALSE;
+gboolean out_tree = FALSE;
 GdaServerProvider *prov_obj = NULL;
 
 static GOptionEntry entries[] = {
         { "provider", 'p', 0, G_OPTION_ARG_STRING, &prov, "Provider name", "provider"},
         { "op", 'o', 0, G_OPTION_ARG_STRING, &op, "Operation", "operation name"},
         { "list-ops", 'l', 0, G_OPTION_ARG_NONE, &list_ops, "List existing operations", NULL },
-        { NULL }
+        { "tree", 't', 0, G_OPTION_ARG_NONE, &out_tree, "Output results as a tree (default is as XML)", NULL },
+        { NULL, 0, 0, 0, NULL, NULL, NULL }
 };
 
 static xmlDocPtr merge_specs (const gchar *xml_dir, GdaServerOperationType type, gboolean *op_supported, GError **error);
@@ -34,14 +37,14 @@ main (int argc, char **argv) {
         }
         g_option_context_free (context);
 
-	gda_init ("list-server-op", PACKAGE_VERSION, argc, argv);
-	xml_dir = gda_gbr_get_file_path (GDA_DATA_DIR, "libgda-3.0", NULL);
+	gda_init ();
+	xml_dir = gda_gbr_get_file_path (GDA_DATA_DIR, "libgda-4.0", NULL);
 	g_print (_("Using XML descriptions in %s\n"), xml_dir);
 	if (prov)
 		g_print ("For provider %s\n", prov);
 
 	if (prov) {
-		prov_obj = gda_config_get_provider_object (prov, &error);
+		prov_obj = gda_config_get_provider (prov, &error);
 		if (!prov_obj) {
 			g_print (_("Could not create provider object: %s\n"), 
 				 error && error->message ? error->message : _("No detail"));
@@ -55,7 +58,7 @@ main (int argc, char **argv) {
 			g_print (_("Existing operation types for provider '%s':\n"), prov);
 		else
 			g_print (_("Existing operation types:\n"));
-		for (type = GDA_SERVER_OPERATION_CREATE_DB; type < GDA_SERVER_OPERATION_NB; type++) {
+		for (type = GDA_SERVER_OPERATION_CREATE_DB; type < GDA_SERVER_OPERATION_LAST; type++) {
 			if (! prov_obj ||
 			    (prov_obj && gda_server_provider_supports_operation (prov_obj, NULL, type, NULL)))
 				g_print ("%s\n", gda_server_operation_op_type_to_string (type));
@@ -64,7 +67,7 @@ main (int argc, char **argv) {
 	}
 
 	GdaServerOperationType type;
-	for (type = GDA_SERVER_OPERATION_CREATE_DB; type != GDA_SERVER_OPERATION_NB; type++) {
+	for (type = GDA_SERVER_OPERATION_CREATE_DB; type != GDA_SERVER_OPERATION_LAST; type++) {
 		xmlDocPtr doc;
 		GError *error = NULL;
 		gboolean op_supported;
@@ -75,12 +78,27 @@ main (int argc, char **argv) {
 		g_print (_("Description for type: %s\n"), gda_server_operation_op_type_to_string (type));
 		doc = merge_specs (xml_dir, type, &op_supported, &error);
 		if (doc) {
-			xmlChar *buf;
-			gint len;
-			xmlKeepBlanksDefault (0);
-			xmlDocDumpFormatMemory (doc, &buf, &len, 1);
-			g_print ("%s\n", buf);
-			xmlFree (buf);
+			if (out_tree) {
+				GdaTree *tree;
+				GdaTreeManager *mgr;
+
+				tree = gda_tree_new ();
+				mgr = gda_tree_mgr_xml_new (xmlDocGetRootElement (doc), "prov_name|id|name|gdatype|node_type|descr");
+				gda_tree_add_manager (tree,  mgr);
+				gda_tree_manager_add_manager (mgr, mgr);
+				g_object_unref (mgr);
+				gda_tree_update_all (tree, NULL);
+				gda_tree_dump (tree, NULL, NULL);
+				g_object_unref (tree);
+			}
+			else {
+				xmlChar *buf;
+				gint len;
+				xmlKeepBlanksDefault (0);
+				xmlDocDumpFormatMemory (doc, &buf, &len, 1);
+				g_print ("%s\n", buf);
+				xmlFree (buf);
+			}
 			xmlFreeDoc (doc);
 		}
 		else {
@@ -103,7 +121,7 @@ remove_all_extra (xmlNodePtr node)
 	xmlNodePtr child;
 	for (child = node->last; child; ) {
 		xmlChar *lang = xmlGetProp (child, (xmlChar*)"lang");
-		if (lang || !strcmp (child->name, "sources")) {
+		if (lang || !strcmp ((gchar *) child->name, "sources")) {
 			/* remove this node */
 			xmlNodePtr prev = child->prev;
 			if (lang)
@@ -152,15 +170,15 @@ make_paths (xmlNodePtr node, const gchar *parent_path, GSList *exist_list)
 		path->path = pstr;
 		retlist = g_slist_append (retlist, path);
 
-		if (!strcmp (node->name, "parameters"))
+		if (!strcmp ((gchar*) node->name, "parameters"))
 			path->node_type = GDA_SERVER_OPERATION_NODE_PARAMLIST;
-		else if (!strcmp (node->name, "parameter"))
+		else if (!strcmp ((gchar*) node->name, "parameter"))
 			path->node_type = GDA_SERVER_OPERATION_NODE_PARAM;
-		else if (!strcmp (node->name, "sequence"))
+		else if (!strcmp ((gchar*) node->name, "sequence"))
 			path->node_type = GDA_SERVER_OPERATION_NODE_SEQUENCE;
-		else if (!strcmp (node->name, "gda_array"))
+		else if (!strcmp ((gchar*) node->name, "gda_array"))
 			path->node_type = GDA_SERVER_OPERATION_NODE_DATA_MODEL;
-		else if (!strcmp (node->name, "gda_array_field")) {
+		else if (!strcmp ((gchar*) node->name, "gda_array_field")) {
 			path->node_type = GDA_SERVER_OPERATION_NODE_DATA_MODEL_COLUMN;
 			g_free (path->path);
 			pstr = g_strdup_printf ("%s/@%s", parent_path, id);
@@ -172,17 +190,17 @@ make_paths (xmlNodePtr node, const gchar *parent_path, GSList *exist_list)
 		xmlChar *prop;
 		prop = xmlGetProp (node, BAD_CAST "name");
 		if (prop) {
-			path->name = g_strdup (prop);
+			path->name = g_strdup ((gchar*) prop);
 			xmlFree (prop);
 		}
 		prop = xmlGetProp (node, BAD_CAST "descr");
 		if (prop) {
-			path->descr = g_strdup (prop);
+			path->descr = g_strdup ((gchar*) prop);
 			xmlFree (prop);
 		}
 		prop = xmlGetProp (node, BAD_CAST "gdatype");
 		if (prop) {
-			path->type = g_strdup (prop);
+			path->type = g_strdup ((gchar*) prop);
 			xmlFree (prop);
 		}
 
@@ -191,7 +209,6 @@ make_paths (xmlNodePtr node, const gchar *parent_path, GSList *exist_list)
 		for (child = node->children; child; child = child->next) 
 			retlist = make_paths (child, pstr, retlist);
 
-
 		xmlFree (id);
 	}
 	else {
@@ -237,11 +254,11 @@ merge_spec_from_root (xmlNodePtr root, const gchar *provider, xmlDocPtr pdoc)
 		/* find node for this path ID, and create it if not yet present */
 		xmlNodePtr node = NULL;
 		for (node = root->children; node; node = node->next) {
-			if (!strcmp (node->name, "path")) {
+			if (!strcmp ((gchar*) node->name, "path")) {
 				xmlChar *pid;
-				pid = xmlGetProp (node, "id");
+				pid = xmlGetProp (node, BAD_CAST "id");
 				if (pid) {
-					if (!strcmp (pid, path->path)) {
+					if (!strcmp ((gchar*) pid, path->path)) {
 						xmlFree (pid);
 						break;
 					}
@@ -275,16 +292,19 @@ merge_spec_from_root (xmlNodePtr root, const gchar *provider, xmlDocPtr pdoc)
 			xmlChar *node_type;
 			node_type = xmlGetProp (node, BAD_CAST "node_type");
 			g_assert (node_type);
-			if (strcmp (node_type, node_type_to_string (path->node_type)))
-				xmlSetProp (pnode, BAD_CAST "node_type", node_type_to_string (path->node_type));
+			if (strcmp ((gchar*) node_type, 
+				    node_type_to_string (path->node_type)))
+				xmlSetProp (pnode, BAD_CAST "node_type", 
+					    BAD_CAST node_type_to_string (path->node_type));
 			xmlFree (node_type);
 			
 			/* check gdatype is similar to "node->gdatype" */
 			node_type = xmlGetProp (node, BAD_CAST "gdatype");
-			if ((node_type && path->type && strcmp (node_type, path->type)) ||
+			if ((node_type && path->type && 
+			     strcmp ((gchar*) node_type, path->type)) ||
 			    (!node_type && path->type) ||
 			    (node_type && *node_type && !path->type))
-				xmlSetProp (pnode, BAD_CAST "gdatype", path->type);
+				xmlSetProp (pnode, BAD_CAST "gdatype", BAD_CAST path->type);
 			if (node_type) 
 				xmlFree (node_type);
 		}
@@ -327,8 +347,12 @@ merge_specs (const gchar *xml_dir, GdaServerOperationType type, gboolean *op_sup
 			provider_end = g_strrstr (provider, suffix);
 			*provider_end = 0;
 			
-			if (lcprov && strcmp (lcprov, provider))
-				continue;
+			if (lcprov && strcmp (lcprov, provider)) {
+				/* handle the name mismatch between the provider named "PostgreSQL" and
+				 * the file names which start with "postgres" */
+				if (strcmp (provider, "postgres") || strcmp (lcprov, "postgresql"))
+					continue;
+			}
 
 			*op_supported = TRUE;
 			pdoc = xmlParseFile (cfile);
@@ -341,7 +365,7 @@ merge_specs (const gchar *xml_dir, GdaServerOperationType type, gboolean *op_sup
 			remove_all_extra (xmlDocGetRootElement (pdoc));
 
 			if (!doc) {
-				doc = xmlNewDoc ("1.0");
+				doc = xmlNewDoc (BAD_CAST "1.0");
 				root = xmlNewNode (NULL, BAD_CAST "server_op");
 				xmlDocSetRootElement (doc, root);
 			}
